--[[
--
--  EkCTRA_MTTargets 1.56
--
--  Refer to index.html for documentation and complete change history.
--
--  Author: Dargen of Eternal Keggers, Terenas.
--          http://www.eternalkeggers.net
--
--  Addons: http://www.wowinterface.com/portal.php?uid=25131
--          http://my.curse.com/dargen/projects/
--
--]]

-- ----------
-- Variables which get saved.
-- ----------
EkCTRA_MTTargets_Status = 1;  -- Status: 0=disabled, 1=enabled
EkCTRA_MTTargets_MTPlayers = 1;  -- Main Tank Players: 0 == disabled, 1 == enabled
EkCTRA_MTTargets_MTPets = 1;  -- Main Tank Pets: 0 == disabled, 1 == enabled
EkCTRA_MTTargets_GroupHasNoPet = 1;  -- Hide MT Pets group when none of MT's have a pet: 0==Show always, 1=Hide when no pet
EkCTRA_MTTargets_PlayerHasNoPet = 1;  -- Hide an MT Player's pet box when they don't have a pet: 0==Show always, 1=Hide when no pet
EkCTRA_MTTargets_AlignToTitle = 1;  -- Align other groups to the dragged group's title (when the dragged group doesn't overlap anything): 0=No, 1=Yes
EkCTRA_MTTargets_GapHide = 0;  -- Hide horizontal gap between MT players/MT Targets/MT Pets: 1=Hide, 0=Show
EkCTRA_MTTargets_LockJoined = 1;  -- Join MT groups: 0=No, 1=Yes.
EkCTRA_MTTargets_WindowPositions = {};  -- Saved window positions
EkCTRA_MTTargets_PlayerBuffs = 1;  -- 0==Don't show player buffs/debuffs, 1==Show player buffs/debuffs
EkCTRA_MTTargets_HideMTAndPlayers = 1;   -- 1==Hide MT Players group when MT Targets group is hidden.
EkCTRA_MTTargets_HideMTAndPets = 1;  -- 1==Hide MT Pets group when MT Targets group is hidden.

-- New saved variables in 1.4
EkCTRA_MTTargets_ShowMTRaidTargetIcons = 1;  -- 0==Hide raid icons on MT Targets and TargetsTargets, 1==Show

-- New saved variables in 1.5
EkCTRA_MTTargets_SidePlayer = 0; -- 0==MT Players on left side of MT Targets, 1==Right side
EkCTRA_MTTargets_SidePet = 0; -- 0==MT Pets on left side of MT Targets, 1==Right side
EkCTRA_MTTargets_SidePetPlayer = 0; -- 0==MT Pets on left side of MT Players, 1==Right side

-- ----------
-- Variables which are not saved.
-- ----------
EkCTRA_MTTargets_cVersion = "1.56";

EkCTRA_MTTargets_Slash1 = "/ekctramt"
EkCTRA_MTTargets_Slash2 = "/ekctra_mttargets"
EkCTRA_MTTargets_Slash3 = "/ekctramttargets"

EkCTRA_MTTargets_InWorld = nil;  -- Gets set to 1 once player is in the world.
EkCTRA_MTTargets_Test = nil; -- nil=Not test mode, 1=Test mode
EkCTRA_MTTargets_AttachedTo = nil;
EkCTRA_MTTargets_DragAll = nil;
EkCTRA_MTTargets_SavePoint= nil;

-- EkCheck configuration for this addon.
EkCTRA_MTTargets_EkCheck_Config = {
	addonName = "EkCTRA_MTTargets",
	funcPrefix = "EkCTRA_MTTargets_EkCheck_",
	defaultColumn = 11,
	columns = {
		{ heading=EkCTRA_MTTargets_TEXT_EkCheck_Version, width=60, show=11, sort=11, order="A", justify="CENTER", red=nil, green=nil, blue=nil },
	},
};

-- The valid MT group names.
--
-- The Players and Pets groups are named so as to take advantage
-- of existing tests done in CT_RaidAssist that compare
-- for the letters "CT_RAMTGroup" at the start of the frame
-- name.  This allows existing CT_RaidAssist code to handle
-- the new frames when possible (eg. CT_RA_MemberFrame_OnEnter)
-- without duplicating that code in this addon.
--
-- This array is not the only location of the individual group names.
-- First element should be the MT Target group, then Players, then Pets.
-- The order of these groups is important (see prepareFrame2()).
EkCTRA_MTTargets_Groups = {
	"CT_RAMTGroup",
	"CT_RAMTGroupPlayer",
	"CT_RAMTGroupPet",
};


-- These are the names of the CT_RaidAssist functions that get hooked.
local EkCTRA_MTTargets_HookedFuncs = {
	{name = "CT_RA_UpdateUnitDebuffs"},
	{name = "CT_RA_UpdateUnitBuffs"},

	{name = "CT_RA_UpdateMTs"},
	{name = "CT_RA_UpdateRaidFrames"},
	{name = "CT_RA_UpdateRaidFrameData"},
	{name = "CT_RA_UpdateRaidFrameOptions"},

	{name = "CT_RAMenuMisc_OnUpdate"},
	{name = "CT_RAMenuAdditional_ScalingMT_OnValueChanged"},

	{name = "CT_RAMTGroupDrag", script = "OnMouseDown", new = "CT_RAMTGroupDrag"},
	{name = "CT_RAMTGroupPlayerDrag", script = "OnMouseDown", new = "CT_RAMTGroupDrag"},
	{name = "CT_RAMTGroupPetDrag", script = "OnMouseDown", new = "CT_RAMTGroupDrag"},

	{name = "CT_RAMTGroupDrag", script = "OnMouseUp", new = "CT_RAMTGroupDrag"},
	{name = "CT_RAMTGroupPlayerDrag", script = "OnMouseUp", new = "CT_RAMTGroupDrag"},
	{name = "CT_RAMTGroupPetDrag", script = "OnMouseUp", new = "CT_RAMTGroupDrag"},

	{name = "CT_RA_UpdateRaidMovability"},
	{name = "CT_RA_UpdateRaidGroupColors"},

	{name = "CT_RAMenu_General_ResetWindows"},
	{name = "CT_RAMenu_SaveWindowPositions"},
	{name = "CT_RAMenu_UpdateWindowPositions"},

	{name = "CT_RA_MemberFrame_OnEnter"},
	{name = "CT_RA_Drag_OnEnter"},

	{name = "CT_RAMenuDisplay_LockGroups"},  -- Hooked to fix a bug
	{name = "CT_RAMenuDisplay_ShowGroupNames"},  -- Hooked to fix a bug
};

-- This is used to save original functions when hooking them.
EkCTRA_MTTargets_OldFunc = {};


-- ------------------------------------------------------------
-- Command line functions.
-- ------------------------------------------------------------


function EkCTRA_MTTargets_Command(msg)
	-- ----------
	-- Examine command line.
	-- ----------
	local pos1, pos2, cmd, rest = string.find(msg, "^%s-(%S+)(.*)");

	local command = "";
	if (cmd) then
		command = string.lower(cmd);
	else
		cmd = "";
	end

	if (command == EkCTRA_MTTargets_TEXT_Command_Help or command == "?" or command == "") then
		EkCTRA_MTTargets_Command_Help();

	elseif (command == EkCTRA_MTTargets_TEXT_Command_Status) then
		EkCTRA_MTTargets_Command_Status();

	elseif (command == EkCTRA_MTTargets_TEXT_Command_Enable) then
		EkCTRA_MTTargets_Command_Enable();

	elseif (command == EkCTRA_MTTargets_TEXT_Command_Disable) then
		EkCTRA_MTTargets_Command_Disable();

	elseif (command == EkCTRA_MTTargets_TEXT_Command_Toggle) then
		EkCTRA_MTTargets_Command_Toggle();

	elseif (command == EkCTRA_MTTargets_TEXT_Command_Players) then
		-- Toggle MT Players group.
		EkCTRA_MTTargets_Command_Players()

	elseif (command == EkCTRA_MTTargets_TEXT_Command_Pets) then
		-- Toggle MT Pets group.
		EkCTRA_MTTargets_Command_Pets()

	elseif (command == "_test") then
		EkCTRA_MTTargets_Command_Test();

	else
		-- Invalid command.
		EkCTRA_MTTargets_InvalidCommand()
	end
end


function EkCTRA_MTTargets_Print(text)
	-- ----------
	-- Print
	-- ----------
	EkCTRA_MTTargets_Print2("<EkCTRA_MTTargets> " .. text);
end


function EkCTRA_MTTargets_Print2(msg)
	-- ----------------
	-- Print a message.
	-- ----------------
	DEFAULT_CHAT_FRAME:AddMessage(msg);
	return;
end


function EkCTRA_MTTargets_Command_Help()
	-- ----------
	-- Help command.
	-- ----------
	local slash = EkCTRA_MTTargets_Slash1 .. " ";
	EkCTRA_MTTargets_Print2(" ");
	EkCTRA_MTTargets_Print2("EkCTRA_MTTargets " .. EkCTRA_MTTargets_cVersion .. " by Dargen, Eternal Keggers, Terenas.");
	EkCTRA_MTTargets_Print2(" ");
	EkCTRA_MTTargets_Print2(slash .. "[" .. EkCTRA_MTTargets_TEXT_Command_Help .. "] -- " .. EkCTRA_MTTargets_TEXT_Help_Help);
	EkCTRA_MTTargets_Print2(slash .. EkCTRA_MTTargets_TEXT_Command_Status .. " -- " .. EkCTRA_MTTargets_TEXT_Help_Status);
	EkCTRA_MTTargets_Print2(slash .. EkCTRA_MTTargets_TEXT_Command_Enable .. " -- " .. EkCTRA_MTTargets_TEXT_Help_Enable);
	EkCTRA_MTTargets_Print2(slash .. EkCTRA_MTTargets_TEXT_Command_Disable .. " -- " .. EkCTRA_MTTargets_TEXT_Help_Disable);
	EkCTRA_MTTargets_Print2(slash .. EkCTRA_MTTargets_TEXT_Command_Toggle .. " -- " .. EkCTRA_MTTargets_TEXT_Help_Toggle);
	EkCTRA_MTTargets_Print2(slash .. EkCTRA_MTTargets_TEXT_Command_Players .. " -- " .. EkCTRA_MTTargets_TEXT_Help_Players);
	EkCTRA_MTTargets_Print2(slash .. EkCTRA_MTTargets_TEXT_Command_Pets .. " -- " .. EkCTRA_MTTargets_TEXT_Help_Pets);
	EkCTRA_MTTargets_Print2(" ");
	EkCTRA_MTTargets_Print2(EkCTRA_MTTargets_TEXT_Help_RightClick);
end


function EkCTRA_MTTargets_Command_Version()
	-- ----------
	-- Version command.
	-- ----------
	EkCTRA_MTTargets_Print(EkCTRA_MTTargets_cVersion);
end


function EkCTRA_MTTargets_Command_Status()
	-- ----------
	-- Status command.
	-- ----------
	EkCTRA_MTTargets_isCTRA(1);
	EkCTRA_MTTargets_ShowStatus();
end


function EkCTRA_MTTargets_Command_Enable()
	-- ----------
	-- Enable command.
	-- ----------
	if (InCombatLockdown()) then
		return;
	end

	EkCTRA_MTTargets_Set_Status(1);
	EkCTRA_MTTargets_ShowStatus();
end


function EkCTRA_MTTargets_Command_Disable()
	-- ----------
	-- Disable command.
	-- ----------
	if (InCombatLockdown()) then
		return;
	end

	EkCTRA_MTTargets_Set_Status(0);
	EkCTRA_MTTargets_ShowStatus();
end


function EkCTRA_MTTargets_Command_Toggle()
	-- ----------
	-- Toggle command (enable/disable).
	-- ----------
	if (InCombatLockdown()) then
		return;
	end

	if (EkCTRA_MTTargets_Status == 0) then
		EkCTRA_MTTargets_Set_Status(1);
	else
		EkCTRA_MTTargets_Set_Status(0);
	end
	EkCTRA_MTTargets_ShowStatus();
end


function EkCTRA_MTTargets_Command_Players()
	-- ----------
	-- Toggle MT Players
	-- ----------
	if (InCombatLockdown()) then
		return;
	end

	EkCTRA_MTTargets_Set_Players(value);
end


function EkCTRA_MTTargets_Command_Pets()
	-- ----------
	-- Toggle MT Pets
	-- ----------
	if (InCombatLockdown()) then
		return;
	end

	EkCTRA_MTTargets_Set_Pets(value);
end


function EkCTRA_MTTargets_Command_Test()
	-- ----------
	-- Test command.
	-- ----------
	if (EkCTRA_MTTargets_Test) then
		EkCTRA_MTTargets_Test = nil;
		EkCTRA_MTTargets_Print("Test mode disabled.");
	else
		EkCTRA_MTTargets_Test = 1;
		EkCTRA_MTTargets_Print("Test mode enabled.");
	end
end


function EkCTRA_MTTargets_InvalidCommand()
	-- --------------
	-- Invalid command message.
	-- --------------
	EkCTRA_MTTargets_Print(EkCTRA_MTTargets_TEXT_InvalidCommand .. " " .. EkCTRA_MTTargets_Slash1 .. " " .. EkCTRA_MTTargets_TEXT_Command_Help);
end


-- ------------------------------------------------------------
-- Status functions.
-- ------------------------------------------------------------


function EkCTRA_MTTargets_ShowStatus()
	-- ----------
	-- Show status.
	-- ----------
	if (EkCTRA_MTTargets_Status == 0) then
		EkCTRA_MTTargets_Print(EkCTRA_MTTargets_TEXT_IsDisabled);
	else
		EkCTRA_MTTargets_Print(EkCTRA_MTTargets_TEXT_IsEnabled);
	end
end


-- ------------------------------------------------------------
-- Configuration functions.
-- ------------------------------------------------------------


function EkCTRA_MTTargets_SetValue(new, current)
	-- ----------
	-- General routine to set value to new value of 0 or 1, 
	-- or to toggle between those two values if new value is nil.
	-- ----------
	local value;

	if (new == nil) then
		if (current == 1) then
			value = 0;
		else
			value = 1;
		end
	elseif (new == 1) then
		value = 1;
	else
		value = 0;
	end

	return value;
end


function EkCTRA_MTTargets_Set_Common(value, varName, combat, update)
	-- ----------
	-- Common function to set a value.
	-- ----------
	if (combat) then
		if (InCombatLockdown()) then
			return;
		end
	end

	setglobal(varName, EkCTRA_MTTargets_SetValue(value, getglobal(varName)));

	if (update) then
		CT_RA_UpdateRaidFrameData();
		CT_RA_UpdateMTs(true);
	end
end


function EkCTRA_MTTargets_Set_Status(value, print)
	-- ----------
	-- Enable/disable this addon.
	-- ----------
	EkCTRA_MTTargets_Status = EkCTRA_MTTargets_SetValue(value, EkCTRA_MTTargets_Status);

	if (print) then
		EkCTRA_MTTargets_ShowStatus();
	end

	if (EkCTRA_MTTargets_Status == 1) then
		-- -----
		-- Enable the addon.
		-- -----

		-- Update the Main Tank Player and Pet frames.
		CT_RA_UpdateRaidFrameData();
		CT_RA_UpdateMTs(true);

		-- Set scaling timer to trigger a quick update.
		CT_RAMenuGlobalFrame.scaleupdate = 0.1;
	else
		-- -----
		-- Disable the addon.
		-- -----

		-- Hide the Players and Pets groups.

		CT_RAMTGroupPlayer:Hide();
		if (CT_RAMTGroupPlayer.GroupName) then
			CT_RAMTGroupPlayer.GroupName:Hide();
		end
		CT_RAMTGroupPlayerDrag:Hide();
		CT_RAMTGroupPlayerMenu:Hide();

		CT_RAMTGroupPet:Hide();
		if (CT_RAMTGroupPet.GroupName) then
			CT_RAMTGroupPet.GroupName:Hide();
		end
		CT_RAMTGroupPetDrag:Hide();
		CT_RAMTGroupPetMenu:Hide();

		CT_RAMTGroupMenu:Hide();

		for i=1, 10 do
			getglobal("EkCTRA_MTTargets_MTOver" .. i):Hide();
			getglobal("EkCTRA_MTTargets_MTTOver" .. i):Hide();
		end

		if (not EkCTRA_MTTargets_isCTRA()) then
			return;
		end

		CT_RA_UpdateRaidFrameData();
		CT_RA_UpdateMTs(true);
	end
end


function EkCTRA_MTTargets_Set_Players(value)
	-- ----------
	-- Enable/disable MT Players group.
	-- ----------
	EkCTRA_MTTargets_Set_Common(value, "EkCTRA_MTTargets_MTPlayers", true, true);
end


function EkCTRA_MTTargets_Set_Pets(value)
	-- ----------
	-- Enable/disable MT Pets group.
	-- ----------
	EkCTRA_MTTargets_Set_Common(value, "EkCTRA_MTTargets_MTPets", true, true);
end

function EkCTRA_MTTargets_Set_PlayerBuffs(value)
	-- ----------
	-- Show or hide player buffs/debuffs in the MT Players group.
	-- ----------
	EkCTRA_MTTargets_PlayerBuffs = EkCTRA_MTTargets_SetValue(value, EkCTRA_MTTargets_PlayerBuffs);

	CT_RA_UpdateMTs(true);
end

function EkCTRA_MTTargets_Set_GroupHasNoPet(value)
	-- ----------
	-- Hide MT Pets group if group has no pet.
	-- ----------
	EkCTRA_MTTargets_Set_Common(value, "EkCTRA_MTTargets_GroupHasNoPet", true, true);
end


function EkCTRA_MTTargets_Set_PlayerHasNoPet(value)
	-- ----------
	-- Hide MT Pets box if player has no pet.
	-- ----------
	EkCTRA_MTTargets_Set_Common(value, "EkCTRA_MTTargets_PlayerHasNoPet", true, true);
end

function EkCTRA_MTTargets_Set_LockJoined(value)
	-- ----------
	-- Join MT groups together.
	-- ----------
	EkCTRA_MTTargets_Set_Common(value, "EkCTRA_MTTargets_LockJoined", true, true);
end

function EkCTRA_MTTargets_Set_SidePlayer(value)
	-- ----------
	-- Side of MT Targets that the MT Player group is on.
	-- ----------
	EkCTRA_MTTargets_Set_Common(value, "EkCTRA_MTTargets_SidePlayer", true, true);
end

function EkCTRA_MTTargets_Set_SidePet(value)
	-- ----------
	-- Side of MT Targets that the MT Pet group is on.
	-- ----------
	EkCTRA_MTTargets_Set_Common(value, "EkCTRA_MTTargets_SidePet", true, true);
end

function EkCTRA_MTTargets_Set_SidePetPlayer(value)
	-- ----------
	-- Side of MT Player that the MT Pet group is on.
	-- ----------
	EkCTRA_MTTargets_Set_Common(value, "EkCTRA_MTTargets_SidePetPlayer", true, true);
end

function EkCTRA_MTTargets_Set_HideMTAndPlayers(value)
	-- ----------
	-- Enable/disable hiding of MT Players group when MT Targets are hidden.
	-- ----------
	EkCTRA_MTTargets_Set_Common(value, "EkCTRA_MTTargets_HideMTAndPlayers", true, true);
end


function EkCTRA_MTTargets_Set_HideMTAndPets(value)
	-- ----------
	-- Enable/disable hiding of MT Pets group when MT Targets are hidden.
	-- ----------
	EkCTRA_MTTargets_Set_Common(value, "EkCTRA_MTTargets_HideMTAndPets", true, true);
end

function EkCTRA_MTTargets_Set_GapHide(value)
	-- ----------
	-- Hide gap between joined MT groups (when frame border is hidden).
	-- ----------
	EkCTRA_MTTargets_Set_Common(value, "EkCTRA_MTTargets_GapHide", true, true);
end

function EkCTRA_MTTargets_Set_AlignToTitle(value)
	-- ----------
	-- Align to MT group title after dragging.
	-- ----------
	EkCTRA_MTTargets_AlignToTitle = EkCTRA_MTTargets_SetValue(value, EkCTRA_MTTargets_AlignToTitle);
end

function EkCTRA_MTTargets_Set_ShowMTRaidTargetIcons(value)
	-- ----------
	-- Show MT raid target icons.
	-- ----------
	EkCTRA_MTTargets_ShowMTRaidTargetIcons = EkCTRA_MTTargets_SetValue(value, EkCTRA_MTTargets_ShowMTRaidTargetIcons);
end


-- ------------------------------------------------------------
-- Initialization functions.
-- ------------------------------------------------------------


function EkCTRA_MTTargets_isCTRA(showErr)
	-- --------------
	-- Test if CT_RaidAssist is present and compatible.
	-- --------------
	local ok = true;
	for i = 1, getn(EkCTRA_MTTargets_HookedFuncs) do
		local f = EkCTRA_MTTargets_HookedFuncs[i];
		if (f.script) then
			frame = getglobal(f.name);
			if (not frame) then
				ok = false;
				break;
			end
			if (not frame:GetScript(f.script)) then
				ok = false;
				break;
			end
		else
			if (not getglobal(f.name)) then
				ok = false;
				break;
			end
		end
	end
	if (ok) then
		return true;
	end

	if (showErr) then
	        -- "CT_RaidAssist not detected/incompatible."
	        EkCTRA_MTTargets_Print(EkCTRA_MTTargets_TEXT_CTRaid_Fail);
	end
        return false;
end


function EkCTRA_MTTargets_OnLoad()
	-- --------------
	-- Loading addon.
	-- --------------
	SLASH_ekctramt1 = EkCTRA_MTTargets_Slash1;
	SLASH_ekctramt2 = EkCTRA_MTTargets_Slash2;
	SLASH_ekctramt3 = EkCTRA_MTTargets_Slash3;
	SlashCmdList["ekctramt"] = function( msg )
		EkCTRA_MTTargets_Command(msg);
	end

	EkCTRA_MTTargets:RegisterEvent("PLAYER_ENTERING_WORLD");
	EkCTRA_MTTargets:RegisterEvent("RAID_ROSTER_UPDATE");
	EkCTRA_MTTargets:RegisterEvent("PLAYER_REGEN_DISABLED");
	EkCTRA_MTTargets:RegisterEvent("PLAYER_REGEN_ENABLED");
	EkCTRA_MTTargets:RegisterEvent("UPDATE_BINDINGS");

--	EkCTRA_MTTargets:RegisterEvent("ADDON_ACTION_BLOCKED");
end


function EkCTRA_MTTargets_OnEvent(event)
	-- --------------
	-- Handle events.
	-- --------------
	if (event == "PLAYER_ENTERING_WORLD") then
		if (not EkCTRA_MTTargets_InWorld) then
			EkCTRA_MTTargets_InWorld = 1;

			-- Raid status window enhancements.
			if (EkCTRA_MTTargets_isCTRA()) then
				local ok = EkCTRA_MTTargets_Init();
				if (not ok) then
					EkCTRA_MTTargets_Print("Error initializing addon.");
				end
			end

			if (EkCheck_RegisterAddon) then
				EkCheck_RegisterAddon(EkCTRA_MTTargets_EkCheck_Config);
			end
		end
	elseif (event == "RAID_ROSTER_UPDATE") then
		-- When initially logging in GetNumRaidMembers() returns 0 even if you are in a raid.
		-- When we get this event and the function returns a non-zeo value, then update the
		-- MT frames, and set a var so we don't do this again until next login.
		if (not EkCTRA_MTTargets_RaidRosterUpdate) then
			if (GetNumRaidMembers() > 0) then
				EkCTRA_MTTargets_Set_Status(EkCTRA_MTTargets_Status);
				EkCTRA_MTTargets_RaidRosterUpdate = 1;
			end
		end
	elseif (event == "PLAYER_REGEN_DISABLED") then
		-- About to enter combat (frames about to be locked down).
		if (not EkCTRA_MTTargets_AttachedTo and EkCTRA_MTTargets_LockJoined == 1) then
			-- While we are in combat lockdown, we will keep the
			-- MT groups attached to each other by their drag frames.
			-- Only one of the visible drag frames can be used to move
			-- the attached groups.
			EkCTRA_MTTargets_AttachAllMTWindows();
		end
	elseif (event == "PLAYER_REGEN_ENABLED") then
		-- Have just left combat (frames are no longer locked down).
		if (EkCTRA_MTTargets_AttachedTo and not EkCTRA_MTTargets_DragAll) then
			EkCTRA_MTTargets_UnattachAllMTWindows();
			CT_RA_UpdateRaidFrameData();
			CT_RA_UpdateMTs(true);
		end
	elseif (event == "UPDATE_BINDINGS") then
		EkCTRA_MTTargets_UpdateBindings();

--[[
	elseif (event == "ADDON_ACTION_BLOCKED") then
		-- arg1 == addon, arg2 == function
		if (arg1 == "EkCTRA_MTTargets") then
			EkCTRA_MTTargets_Print(event .. ", " .. arg1 .. ", " .. (arg2 or "nil"));
		else
			if (strsub(arg2, 1, 18) == "CT_RAMTGroupPlayer") or
			   (strsub(arg2, 1, 15) == "CT_RAMTGroupPet") or
			   (strsub(arg2, 1, 16) == "CT_RAMTGroupMenu") then

				EkCTRA_MTTargets_Print(event .. ", " .. arg1 .. ", " .. (arg2 or "nil"));
			end
		end
]]

	end
end


function EkCTRA_MTTargets_Init()
	-- ----------
	-- Initialize
	-- ----------

	-- Hook some CT_RaidAssist functions.
	local ok = true;
	for i = 1, getn(EkCTRA_MTTargets_HookedFuncs) do
		local f = EkCTRA_MTTargets_HookedFuncs[i];
		ok = EkCTRA_MTTargets_Hook(f.name, f.script, f.new);
		if (not ok) then
			return false;
		end
	end

	local tempOptions = CT_RAMenu_Options["temp"];
	local newScaling = CT_RAMenu_Options["temp"]["MTScaling"];
	local mtwidth, ptwidth, height = CT_RA_GetMTFrameData();

	CT_RAMTGroupPlayer:SetScale(newScaling);
	CT_RAMTGroupPet:SetScale(newScaling);

	CT_RAMTGroupPlayerDrag:SetWidth(mtwidth*newScaling); -- +(22*newScaling));
	CT_RAMTGroupPlayerDrag:SetHeight(height*newScaling/2);

	CT_RAMTGroupPlayerMenu:SetWidth(mtwidth*newScaling); -- +(22*newScaling));
	CT_RAMTGroupPlayerMenu:SetHeight(height*newScaling/2);

	CT_RAMTGroupPetDrag:SetWidth(mtwidth*newScaling); -- +(22*newScaling));
	CT_RAMTGroupPetDrag:SetHeight(height*newScaling/2);

	CT_RAMTGroupPetMenu:SetWidth(mtwidth*newScaling); -- +(22*newScaling));
	CT_RAMTGroupPetMenu:SetHeight(height*newScaling/2);

	CT_RAMTGroupMenu:SetWidth(CT_RAMTGroupDrag:GetWidth());
	CT_RAMTGroupMenu:SetHeight(CT_RAMTGroupDrag:GetHeight());

	-- Set status
	EkCTRA_MTTargets_Set_Status(EkCTRA_MTTargets_Status);

	return true;
end

function EkCTRA_MTTargets_Hook(name, script, new)
	-- ----------
	-- Hook a CT_RaidAssist function or frame:script.
	-- ----------
	local oldObject, oldScript;
	local newObject;

	if (not new) then
		new = name;
	end

	oldObject = getglobal(name);
	if (not oldObject) then
		return false;
	end

	if (script) then
		oldScript = oldObject:GetScript(script);
		if (not oldScript) then
			return false;
		end
		newObject = getglobal("EkCTRA_MTTargets_" .. new .. script);
		if (not newObject) then
			return false;
		end
	else
		newObject = getglobal("EkCTRA_MTTargets_" .. new);
		if (not newObject) then
			return false;
		end
	end

	if (script) then
		EkCTRA_MTTargets_OldFunc[name .. script] = oldScript;
		oldObject:SetScript(script, newObject)
	else
		EkCTRA_MTTargets_OldFunc[name] = oldObject;
		setglobal(name, newObject);
	end

	return true;
end


function EkCTRA_MTTargets_UpdateBindings()
	-- ----------
	-- Update key bindings.
	-- ----------
	local key;
	for i = 1, 10, 1 do
		key = GetBindingKey("EKCTRA_MTTARGETS_MTPLAYER" .. i);
		if (key) then
			SetOverrideBindingClick(CT_RAFrame, false, key, "CT_RAMTGroupPlayerUnitButton" .. i);
		end
		key = GetBindingKey("EKCTRA_MTTARGETS_MTPET" .. i);
		if (key) then
			SetOverrideBindingClick(CT_RAFrame, false, key, "CT_RAMTGroupPetUnitButton" .. i);
		end
	end
end


-- --------
-- EkCheck interface routines
-- --------

function EkCTRA_MTTargets_EkCheck_CreateTooltipText(index, total, data)
	-- ----------
	-- User moved pointer over a line in the list.
	--
	-- index -- position of the item in the list
	-- total -- number of items in the list
	-- data -- Array of data
	--
	-- 	data[]
	-- 	---------------
	-- 	[1] == EkCheck: Player name
	-- 	[2] == EkCheck: Uppercase english class string (ie. non-localized) (eg. "PRIEST")
	-- 	[3] == EkCheck: 0==Addon not loaded, 1==Addon loaded.
	-- 	[4] == EkCheck: 0==Player has not responded to the check, 1==Player has responded to the check.
	-- 	[5] == EkCheck: Player class (localized)
	-- 	[6] to [10] == EkCheck: Reserved
	--
	-- 	[11] == EkCTRA_MTTargets: Version string
	--
	-- Returns:
	--	a) nil -- if no tooltip should be generated.
	--	b) String containing text to appear in the tooltip.
	-- ----------
--	local text;
--	local white = "|c00FFFFFF";
--	return text;

	return nil;
end

function EkCTRA_MTTargets_EkCheck_CreateResponse(data)
	-- ----------------
	-- Create addon specific portion of response string.
	--
	-- data -- Array of data provided by EkCheck.
	--
	-- 	data[]
	-- 	--------
	-- 	[1] == Name of the player that initiated the check.
	-- 	[2] == Name of the addon being checked for.
	--
	-- Returns:
	-- 	a) nil or "" == No addon specific response string.
	--	b) Addon specific response string.
	-- ----------------
	local resp;

	-- Version
	resp = "v" .. EkCTRA_MTTargets_cVersion;

	-- Return the response string.
	return resp;
end

function EkCTRA_MTTargets_EkCheck_ParseResponse(text, data)
	-- ----------------
	-- Parse response text.
	--
	-- text -- The addon specific portion of the response.
	-- data -- Array of data provided by EkCheck.
	--
	-- 	data[]
	-- 	---------------
	-- 	[1] == Player name
	-- 	[2] == Uppercase english class string (ie. non-localized) (eg. "PRIEST")
	-- 	[3] == 0==Addon not loaded, 1==Addon loaded.
	-- 	[4] == 0==Player has not responded to the check, 1==Player has responded to the check.
	-- 	[5] == Player class (localized)
	--
	-- Returns:
	--	a) nil -- Invalid response
	--      b) Array containing addon specific data (to be used for the check window, tooltip, etc).
	-- ----------------
	local msg;
	local version;

	version = "?";

	msg = strlower(text);

	for k, v in string.gmatch(msg, "(%a)(%S+)") do
		if (k == "v") then
			version = v;
		end
	end

	return { version };
end

-- ------------------------------------------------------------
-- Operating functions.
-- ------------------------------------------------------------

function EkCTRA_MTTargets_UnitName(unit)
	local name;
	if (UnitExists(unit)) then
		name = UnitName(unit);
	end
	return name;
end

function EkCTRA_MTTargets_GetShowStatus()
	-- ----------
	-- Determine which MT groups to display (targets, players, pets).
	--
	-- Returns 3 values:
	--   1) Show targets: 1==Yes, nil==No.
	--   2) Show players: 1==Yes, nil==No.
	--   3) Show pets: 1==Yes, nil==No.
	-- ----------
	local showmts, showmtplayers, showmtpets;
	local tempOptions = CT_RAMenu_Options["temp"];
	local numRaidMembers = GetNumRaidMembers();

	-- Are there any main tanks?
	local havemts = false;
	for i = 1, ( tempOptions["ShowNumMTs"] or 10 ), 1 do
		if ( CT_RA_MainTanks[i] ) then
			havemts = true;
			break;
		end
	end

	if (EkCTRA_MTTargets_Status == 1) then
		if (numRaidMembers > 0 and havemts) then
			if (not tempOptions["HideMTs"]) then
				showmts = 1;
			end
			if (EkCTRA_MTTargets_MTPlayers == 1) then
				showmtplayers = 1;
				if (not showmts and EkCTRA_MTTargets_HideMTAndPlayers == 1) then
					showmtplayers = nil;
				end
			end
			if (EkCTRA_MTTargets_MTPets == 1) then
				showmtpets = 1;
				if (not showmts and EkCTRA_MTTargets_HideMTAndPets == 1) then
					showmtpets = nil;
				end
			end
		end
	end

	if (showmtpets and EkCTRA_MTTargets_GroupHasNoPet == 1) then
		-- Hide MT Pets group if none of the main tanks has a pet.
		local petfound;
		local showNumTanks = ( tempOptions["ShowNumMTs"] or 10 );

		if (havemts and showNumTanks > 0) then
			local raidid, name;
			for i = 1, numRaidMembers, 1 do
				raidid = nil;
				name = EkCTRA_MTTargets_UnitName("raid" .. i);
				for k, v in pairs(CT_RA_MainTanks) do
					if (v and v == name) then
						raidid = "raidpet" .. i;
						break;
					end
				end
				if (raidid and UnitExists(raidid) and strlen(EkCTRA_MTTargets_UnitName(raidid) or "") > 0 ) then
					petfound = 1;
					break;
				end
			end
		end

		if (not petfound) then
			-- Hide the MT Pets group
			showmtpets = nil;
		end
	end

	return showmts, showmtplayers, showmtpets;
end


function EkCTRA_MTTargets_Update_AlignToTitle(Group)
	-- ----------
	-- Line up the MT windows vertically so that their
	-- titles are at the same Y value as the specified group.
	-- ----------
	local tempOptions = CT_RAMenu_Options["temp"];

	if ( tempOptions["LockGroups"] ) then
		return;
	end

	if (not Group) then
		Group = getglobal(this:GetName());
	end

	local X, Y = Group:GetLeft(), Group:GetTop();

	if (not X or not Y) then
		return;
	end

	if (InCombatLockdown()) then
		return;
	end

	for key, cGroup in ipairs(EkCTRA_MTTargets_Groups) do
		local cDrag = cGroup .. "Drag";
		if ( cDrag ~= this:GetName() ) then
			local oGroup = getglobal(cDrag);
			local oX, oY = oGroup:GetLeft(), oGroup:GetTop();
			if ( oX and Y ) then
				oGroup:ClearAllPoints();
				oGroup:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", oX, Y-UIParent:GetTop());
			end
		end
	end
end


function EkCTRA_MTTargets_UpdateMenuFrames()
	-- ----------
	-- Hide/unhide the frames used to trap right clicks in order to display
	-- a menu when the drag frames are hidden.
	-- ----------
	if (InCombatLockdown()) then
		return;
	end

	local showmts, showmtplayers, showmtpets = EkCTRA_MTTargets_GetShowStatus();

	-- Show/Hide the special MTGroup frame.
	-- This frame sits at same position as CT_RAMTGroupDrag.
	-- I needed it to be able to trap right mouse buttons when
	-- all windows are locked and the drag frames are hidden.
	if (CT_RAMTGroupDrag:IsShown() or not showmts) then
		CT_RAMTGroupMenu:Hide();
	else
		CT_RAMTGroupMenu:Show();
		CT_RAMTGroupMenu:SetWidth(CT_RAMTGroupDrag:GetWidth());
		CT_RAMTGroupMenu:SetHeight(CT_RAMTGroupDrag:GetHeight());
	end

	if (CT_RAMTGroupPlayerDrag:IsShown() or not showmtplayers) then
		CT_RAMTGroupPlayerMenu:Hide();
	else
		CT_RAMTGroupPlayerMenu:Show();
	end

	if (CT_RAMTGroupPetDrag:IsShown() or not showmtpets) then
		CT_RAMTGroupPetMenu:Hide();
	else
		CT_RAMTGroupPetMenu:Show();
	end
end

function EkCTRA_MTTargets_AttachAllMTWindows(attachTo)
	-- ----------
	-- Attach all the MT groups together by their drag frames.
	-- ----------
	if (InCombatLockdown()) then
		return;
	end

	if (not attachTo) then
		-- Use the first drag frame that is not hidden.
		for key, cGroup in ipairs(EkCTRA_MTTargets_Groups) do
			local cDrag = cGroup .. "Drag";
			local oDrag = getglobal(cDrag);
			if (oDrag:IsShown()) then
				attachTo = oDrag;
				break;
			end
		end
	end

	if (not attachTo) then
		return false;
	end

	local cThisDrag = attachTo:GetName();

	local nThisDragX, nThisDragY = attachTo:GetLeft(), attachTo:GetTop();
	if ( not nThisDragX or not nThisDragY ) then
		return false;
	end

	EkCTRA_MTTargets_AttachedTo = { attachTo };

	for key, cGroup in ipairs(EkCTRA_MTTargets_Groups) do
		local cDrag = cGroup .. "Drag";

		if ( cDrag ~= cThisDrag ) then
			local oDrag = getglobal(cDrag);
			local nDragX, nDragY = oDrag:GetLeft(), oDrag:GetTop();
			if ( nDragX and nDragY ) then
				oDrag:ClearAllPoints();
				oDrag:SetPoint("TOPLEFT", cThisDrag, "TOPLEFT", nDragX - nThisDragX, nDragY - nThisDragY);

				table.insert(EkCTRA_MTTargets_AttachedTo, oDrag);
			end
		end
	end

	return true;
end

function EkCTRA_MTTargets_UnattachAllMTWindows()
	-- ----------
	-- Unattach all the MT groups.
	-- ----------
	if (not EkCTRA_MTTargets_AttachedTo) then
		return false;
	end

	local cThisDrag = EkCTRA_MTTargets_AttachedTo[1]:GetName();

	for key, cGroup in ipairs(EkCTRA_MTTargets_Groups) do
		local cDrag = cGroup .. "Drag";
		if ( cDrag ~= cThisDrag ) then
			local oDrag = getglobal(cDrag);
			local nDragX, nDragY = oDrag:GetLeft(), oDrag:GetTop();
			if ( nDragX and nDragY ) then
				oDrag:ClearAllPoints();
				oDrag:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", nDragX, nDragY - UIParent:GetTop());
			end
		end
	end

	EkCTRA_MTTargets_AttachedTo = nil;
	return true;
end


-- ------------------------------------------------------------
-- Hooked function: CT_RA_UpdateUnitBuffs (from CT_RaidAssist.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RA_UpdateUnitBuffs(buffs, frame, nick)
	-- ----------
	-- This function gets called instead of CT_RA_UpdateUnitBuffs().
	-- ----------

	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RA_UpdateUnitBuffs) then
		EkCTRA_MTTargets_OldFunc.CT_RA_UpdateUnitBuffs(buffs, frame, nick);
	end

	if (EkCTRA_MTTargets_Status ~= 1) then
		return;
	end

	if (EkCTRA_MTTargets_MTPlayers ~= 1) then
		return;
	end

	-- If this person is a main tank, then update the buffs on their Main Tank Player frame.
	local tempOptions = CT_RAMenu_Options["temp"];
	local num, val = 0;
	for i = 1, ( tempOptions["ShowNumMTs"] or 10 ), 1 do
		val = CT_RA_MainTanks[i];
		if ( val ) then
			num = num + 1;
			local frame = getglobal("CT_RAMTGroupPlayerUnitButton"..num);
			if ( frame ) then
				local raidid = SecureButton_GetUnit(frame);  -- frame.unit;
				if ( raidid ) then
					local name = EkCTRA_MTTargets_UnitName(raidid);
					if (name == nick) then
						EkCTRA_MTTargets_MTP_UpdateUnitBuffs(buffs, frame, name);
					end
				end
			end
		end
	end
end

function EkCTRA_MTTargets_MTP_UpdateUnitBuffs(buffs, frame, name)
	-- ----------
	-- Call CT_RA_UpdateUnitBuffs() for a Main Tank Player frame.
	-- ----------
	local tempOptions = CT_RAMenu_Options["temp"];

	if (EkCTRA_MTTargets_PlayerBuffs ~= 1) then
		-- Hide the buff and debuffs
		for j = 1, 4 do
			frame["BuffButton" .. j]:Hide();
		end
		for j = 1, 2 do
			frame["DebuffButton" .. j]:Hide();
		end
		return;
	end

	-- Temporarily pretend that the MT Players group is group 1, and that the option to show group 1 is enabled.
	-- This saves me from having to make a copy of the CT_RA_UpdateUnitBuffs() function and modifying it to work
	-- with the MT Players group.
	local saveparent = frame.frameParent;
	frame.frameParent = {};
	frame.frameParent.id = 1;

	local savegroup = tempOptions["ShowGroups"][frame.frameParent.id];
	tempOptions["ShowGroups"][frame.frameParent.id] = 1;

	if (EkCTRA_MTTargets_OldFunc.CT_RA_UpdateUnitBuffs) then
		EkCTRA_MTTargets_OldFunc.CT_RA_UpdateUnitBuffs(buffs, frame, name);
	end

	-- Restore the original value.
	tempOptions["ShowGroups"][frame.frameParent.id] = savegroup;
	frame.frameParent = saveparent;
end


-- ------------------------------------------------------------
-- Hooked function: CT_RA_UpdateUnitDebuffs (from CT_RaidAssist.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RA_UpdateUnitDebuffs(debuffs, frame)
	-- ----------
	-- This function gets called instead of CT_RA_UpdateUnitDebuffs().
	-- ----------

	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RA_UpdateUnitDebuffs) then
		EkCTRA_MTTargets_OldFunc.CT_RA_UpdateUnitDebuffs(debuffs, frame);
	end

	if (EkCTRA_MTTargets_Status ~= 1) then
		return;
	end

	if (EkCTRA_MTTargets_MTPlayers ~= 1) then
		return;
	end

	-- If this person is a main tank, then update their Main Tank Player frame.
	if (not frame or not frame.id) then
		return;
	end
	local raidid = "raid" .. frame.id;
	local nick = EkCTRA_MTTargets_UnitName(raidid);

	local tempOptions = CT_RAMenu_Options["temp"];
	local num, val = 0;
	for i = 1, ( tempOptions["ShowNumMTs"] or 10 ), 1 do
		val = CT_RA_MainTanks[i];
		if ( val ) then
			num = num + 1;
			local frame = getglobal("CT_RAMTGroupPlayerUnitButton"..num);
			if ( frame ) then
				local raidid = SecureButton_GetUnit(frame);  -- frame.unit;
				if ( raidid ) then
--					local mtid = raidid:match("^(%a+%d+)");
					local name = EkCTRA_MTTargets_UnitName(raidid);
					if (name == nick) then
						EkCTRA_MTTargets_MTP_UpdateUnitDebuffs(debuffs, frame);
					end
				end
			end
		end
	end
end

function EkCTRA_MTTargets_MTP_UpdateUnitDebuffs(debuffs, frame)
	-- ----------
	-- Call CT_RA_UpdateUnitDebuffs() for a Main Tank Player frame.
	-- ----------
	local tempOptions = CT_RAMenu_Options["temp"];

	if (EkCTRA_MTTargets_PlayerBuffs ~= 1) then
		-- Hide the debuffs
		for j = 1, 2 do
			frame["DebuffButton" .. j]:Hide();
		end
		return;
	end

	-- Temporarily pretend that the MT Pets group is group 1, and that the option to show group 1 is enabled.
	-- This saves me from having to make a copy of the CT_RA_UpdateUnitBuffs() function and modifying it to work
	-- with the MT Pets group.
	local saveparent = frame.frameParent;
	frame.frameParent = {};
	frame.frameParent.id = 1;

	local savegroup = tempOptions["ShowGroups"][frame.frameParent.id];
	tempOptions["ShowGroups"][frame.frameParent.id] = 1;

	if (EkCTRA_MTTargets_OldFunc.CT_RA_UpdateUnitDebuffs) then
		EkCTRA_MTTargets_OldFunc.CT_RA_UpdateUnitDebuffs(debuffs, frame);
	end

	-- Restore the original value.
	tempOptions["ShowGroups"][frame.frameParent.id] = savegroup;
	frame.frameParent = saveparent;
end






function EkCTRA_MTTargets_UpdateRaidTargetIcon(mtnum, raidid, cFrame)
	-- -----
	-- Update a frame's raid target icon.
	-- -----
	local icon = getglobal(cFrame .. mtnum .. "Icon");
	local info, index;

	if (UnitExists(raidid) and EkCTRA_MTTargets_ShowMTRaidTargetIcons == 1) then
		index = GetRaidTargetIndex(raidid);
		if (index) then
			info = UnitPopupButtons["RAID_TARGET_" .. index];
		end
	end

	if (info) then
		icon:SetTexture(info.icon);
		if (info.tCoordLeft) then
			icon:SetTexCoord(info.tCoordLeft, info.tCoordRight, info.tCoordTop, info.tCoordBottom);
		end
		icon:Show();
	else
		icon:Hide();
	end
end




-- ------------------------------------------------------------
-- Hooked function: CT_RA_UpdateMTs (from CT_RaidAssist.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RA_UpdateMTs(forceUpdate)
	-- ----------
	-- This function gets called instead of CT_RA_UpdateMTs().
	-- ----------

	-- Call the original function first
	if (EkCTRA_MTTargets_OldFunc.CT_RA_UpdateMTs) then
		EkCTRA_MTTargets_OldFunc.CT_RA_UpdateMTs(forceUpdate);
	end

	if (EkCTRA_MTTargets_Status == 1) then
		-- Handle the Main Tank Players and Pets
		EkCTRA_MTTargets_EK_RA_UpdateMTPs(forceUpdate);
	end
end

function EkCTRA_MTTargets_EK_RA_UpdateMTPs(forceUpdate)
	-- ----------
	-- Update the Main Tank Players.
	-- This is a heavily modified copy of CT_RA_UpdateMTs().
	-- ----------
	EkCTRA_MTTargets_UpdateMenuFrames();

	for i = 1, 10 do
		getglobal("EkCTRA_MTTargets_MTOver" .. i):Hide();
		getglobal("EkCTRA_MTTargets_MTTOver" .. i):Hide();
	end

	local tempOptions = CT_RAMenu_Options["temp"];
	local CT_RA_MainTanks = CT_RA_MainTanks;
	local num, val = 0;
	for i = 1, ( tempOptions["ShowNumMTs"] or 10 ), 1 do
		val = CT_RA_MainTanks[i];
		if ( val ) then
			num = num + 1;

			if (EkCTRA_MTTargets_MTPlayers == 1) then
				local frame = getglobal("CT_RAMTGroupPlayerUnitButton"..num);
				if ( frame and frame:IsShown()) then
					local frameParent = frame.frameParent;
					local raidid = SecureButton_GetUnit(frame);  -- frame.unit;
					if ( raidid ) then
						local mtid = raidid:match("^(%a+%d+)");

						local name = (EkCTRA_MTTargets_UnitName(raidid) or "");
						local hpmax = UnitHealthMax(raidid);
						local mpmax = UnitManaMax(raidid);
						local hppercent;
						local mppercent;
						if (hpmax and hpmax ~= 0) then
							hppercent = UnitHealth(raidid)/hpmax;
						else
							hppercent = -1;
						end
						if (mpmax and mpmax ~= 0) then
							mppercent = UnitMana(raidid)/mpmax;
						else
							mppercent = -1;
						end

						local stats, isFD, isDead, isAFK, isRessed, isOnline;
						stats, isFD, isDead = CT_RA_Stats[name], false, false;
						if ( UnitIsGhost(raidid) or UnitIsDead(raidid) ) then
							isFD = CT_RA_CheckFD(name, raidid)
							isDead = ( isFD == 0 );
						end
						if (stats) then
							isAFK = (stats["AFK"] ~= nil);
							isRessed = stats["isRessed"] or false;
						else
							isAFK = false;
							isRessed = false;
						end
						isOnline = UnitIsConnected(raidid) or false;

						if ( 
							forceUpdate
							or name ~= ( frame.unitName or "" )
							or hppercent ~= ( frame.hppercent or -1 )
							or mppercent ~= ( frame.mppercent or -1 )
							or isOnline ~= frame.ekisOnline  -- not UnitIsConnected(raidid)
							or isFD ~= frame.ekisFD
							or isDead ~= frame.ekisDead
							or isAFK ~= frame.ekisAFK
							or isRessed ~= frame.ekisRessed
						) then
--[[
							if ( not isOnline ) then  -- UnitIsConnected(raidid) ) then
								frame.unitName = nil;
								frame.hppercent = nil;
								frame.mppercent = nil;
								frame.ekisOnline = nil;
								frame.ekisFD = nil;
								frame.ekisDead = nil;
								frame.ekisAFK = nil;
								frame.ekisRessed = nil;
							else
]]
								frame.unitName = name;
								frame.hppercent = hppercent;
								frame.mppercent = mppercent;
								frame.ekisOnline = isOnline;
								frame.ekisFD = isFD;
								frame.ekisDead = isDead;
								frame.ekisAFK = isAFK;
								frame.ekisRessed = isRessed;
--							end
							EkCTRA_MTTargets_EK_RA_UpdateMTP(raidid, mtid, frame, key, val, false);
						end
					end
				end
			end

			if (EkCTRA_MTTargets_MTPets == 1) then
				local frame = getglobal("CT_RAMTGroupPetUnitButton"..num);
				if ( frame and frame:IsShown()) then
					local frameParent = frame.frameParent;
					local raidid = SecureButton_GetUnit(frame);  -- frame.unit;
					if ( raidid ) then
						local mtid = raidid:match("^(%a+%d+)");

						local name = (EkCTRA_MTTargets_UnitName(raidid) or "");
						local hpmax = UnitHealthMax(raidid);
						local mpmax = UnitManaMax(raidid);
						local hppercent;
						local mppercent;
						if (hpmax and hpmax ~= 0) then
							hppercent = UnitHealth(raidid)/hpmax;
						else
							hppercent = -1;
						end
						if (mpmax and mpmax ~= 0) then
							mppercent = UnitMana(raidid)/mpmax;
						else
							mppercent = -1;
						end

						local isDead, isOnline;
						if ( UnitIsGhost(raidid) or UnitIsDead(raidid) ) then
							isDead = true;
						else
							isDead = false;
						end
						isOnline = UnitIsConnected(raidid) or false;

						if ( 
							forceUpdate
							or name ~= ( frame.unitName or "" )
							or hppercent ~= ( frame.hppercent or -1 )
							or mppercent ~= ( frame.mppercent or -1 ) 
							or isOnline ~= frame.ekisOnline  -- not UnitIsConnected(raidid)
							or isDead ~= frame.ekisDead
						) then
--[[
							if ( not isOnline ) then  -- UnitIsConnected(raidid) ) then
								frame.unitName = nil;
								frame.hppercent = nil;
								frame.mppercent = nil;
								frame.ekisOnline = nil;
								frame.ekisDead = nil;
							else
]]
								frame.unitName = name;
								frame.hppercent = hppercent;
								frame.mppercent = mppercent;
								frame.ekisOnline = isOnline;
								frame.ekisDead = isDead;
--							end
							EkCTRA_MTTargets_EK_RA_UpdateMTP(raidid, mtid, frame, key, val, true);
						end
					end
				end
			end

			if (EkCTRA_MTTargets_ShowMTRaidTargetIcons == 1) then
				-- Show MT Target raid icon
				local frame = getglobal("CT_RAMTGroupUnitButton"..num);
				if ( frame ) then
					local cFrame = "EkCTRA_MTTargets_MTOver";
					local raidid = SecureButton_GetUnit(frame); -- frame.unit;
					if ( raidid ) then
						EkCTRA_MTTargets_UpdateRaidTargetIcon(num, raidid, cFrame);
						local f = getglobal(cFrame .. num);
						f:ClearAllPoints();
						f:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -5, -5);
						f:Show();
					end
				end

				-- Show MT Target Target raid icon
				local frame = getglobal("CT_RAMTTGroupUnitButton"..num);
				if ( frame ) then
					local cFrame = "EkCTRA_MTTargets_MTTOver";
					local raidid = SecureButton_GetUnit(frame); -- frame.unit;
					if ( raidid ) then
						EkCTRA_MTTargets_UpdateRaidTargetIcon(num, raidid, cFrame);
						local f = getglobal(cFrame .. num);
						f:ClearAllPoints();
						f:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -5, -5);
						f:Show();
					end
				end
			end
		end
	end
end

function EkCTRA_MTTargets_EK_RA_UpdateMTP(raidid, mtid, frame, key, val, isPet)
	-- ----------
	-- Update a Main Tank Player or Pet
	-- ----------

	-- Call a modified copy of the standard CT_RA_UpdateMT() to udpate name, health, mana, height, width, etc.
	EkCTRA_MTTargets_EK_RA_UpdateMT(raidid, mtid, frame, key, val, isPet);

	if (isPet) then
		return;
	end

	if (EkCTRA_MTTargets_MTPlayers ~= 1) then
		return;
	end

	-- Update the Main Tank Players buffs and debuffs.
	local id = frame.id;
	if ( not id ) then
		return;
	end
	local name = EkCTRA_MTTargets_UnitName("raid" .. id);
	local stats = CT_RA_Stats[name];
	if ( stats ) then
		EkCTRA_MTTargets_MTP_UpdateUnitBuffs(stats["Buffs"], frame, name);
	end
end

function EkCTRA_MTTargets_EK_RA_UpdateMT(raidid, mtid, frame, key, val, isPet)
	-- ----------
	-- Update a Main Tank Player/Pet's health, etc.
	-- This is a modified copy of CT_RA_UpdateMT() (from CT_RaidAssist.lua)
	-- ----------
	local tempOptions = CT_RAMenu_Options["temp"];
	local frameName = frame.name;
	local mtwidth, ptwidth, height = CT_RA_GetMTFrameData();

	if (not InCombatLockdown()) then
		frame:SetWidth(mtwidth); frame:SetHeight(height);
	end

	local defaultColors = tempOptions.DefaultColor;
	frame:SetBackdropColor(defaultColors.r, defaultColors.g, defaultColors.b, defaultColors.a);

	if ( tempOptions["HideBorder"] ) then
		frame:SetBackdropBorderColor(1, 1, 1, 0);
		frame.HPBar:SetPoint("TOPLEFT", frameName, "TOPLEFT", 10, -19);
		frame.HPBG:SetPoint("TOPLEFT", frameName, "TOPLEFT", 10, -19);
		frame.Percent:SetPoint("TOP", frameName, "TOPLEFT", 47, -16);
	else
		frame:SetBackdropBorderColor(1, 1, 1, 1);
		frame.HPBar:SetPoint("TOPLEFT", frameName, "TOPLEFT", 10, -22);
		frame.HPBG:SetPoint("TOPLEFT", frameName, "TOPLEFT", 10, -22);
		frame.Percent:SetPoint("TOP", frameName, "TOPLEFT", 47, -18);
	end
	if ( raidid and UnitExists(raidid) and strlen(EkCTRA_MTTargets_UnitName(raidid) or "") > 0 ) then
		local health, healthmax, mana, manamax = UnitHealth(raidid), UnitHealthMax(raidid), UnitMana(raidid), UnitManaMax(raidid);

		frame.Name:SetHeight(15);
		frame.Status:Hide();
		frame.HPBar:Show();
		frame.HPBG:Show();
-- &&		frame.MPBar:Show();
-- &&		frame.MPBG:Show();
		frame.Name:Show();

		local manaType = UnitPowerType(raidid);
		if ( ( manaType == 0 and not tempOptions["HideMP"] ) or ( manaType > 0 and not tempOptions["HideRP"] and (UnitIsPlayer(raidid) or isPet) ) ) then
			local manaTbl = ManaBarColor[manaType];
			frame.MPBar:SetStatusBarColor(manaTbl.r, manaTbl.g, manaTbl.b);
			frame.MPBG:SetVertexColor(manaTbl.r, manaTbl.g, manaTbl.b, tempOptions["BGOpacity"]);
			frame.MPBar:SetMinMaxValues(0, manamax);
			frame.MPBar:SetValue(mana);
-- &&		else
-- &&			frame.MPBar:Hide();
-- &&			frame.MPBG:Hide();
		end

		local _, class = UnitClass(raidid);
		-- && This "if...end" copied from CT_RA_UpdateUnitDead() and modified.
		if ( ( ( class == CT_RA_WARRIOR or class == CT_RA_ROGUE ) and not tempOptions["HideRP"] ) or ( class ~= CT_RA_WARRIOR and class ~= CT_RA_ROGUE and not tempOptions["HideMP"] ) ) then
			frame.MPBar:Show();
			frame.MPBG:Show();
-- &&			if ( canShowInfo ) then
-- &&				frame.Percent:Show();
-- &&			else
-- &&				frame.Percent:Hide();
-- &&			end
		else
			frame.MPBar:Hide();
			frame.MPBG:Hide();
		end

		frame.Status:Hide();

		local stats, isFD, isDead;
		if (not isPet) then
			local name = (EkCTRA_MTTargets_UnitName(raidid));
			stats, isFD, isDead = CT_RA_Stats[name], false, false;
			if ( UnitIsGhost(raidid) or UnitIsDead(raidid) ) then
				isFD = CT_RA_CheckFD(name, raidid)
				if ( isFD == 0 ) then
					isDead = 1;
				end
			end
		else
			if ( UnitIsGhost(raidid) or UnitIsDead(raidid) ) then
				isDead = 1;
			end
		end

		if ( not UnitIsConnected(raidid) ) then
			frame.HPBar:Hide();
			frame.HPBG:Hide();
			frame.Percent:Hide();
			frame.MPBar:Hide();
			frame.MPBG:Hide();
			frame.Status:Show();
			frame.Status:SetText("OFFLINE");
			frame.status = "offline";

		elseif ( isFD == 1 or isFD == 2 ) then
			frame.HPBar:Hide();
			frame.HPBG:Hide();
			frame.Percent:Hide();
			frame.MPBar:Hide();
			frame.MPBG:Hide();
			frame.Status:Show();
			if ( isFD == 1 ) then
				frame.Status:SetText("Feign Death");
				frame.status = "feigndeath";
			elseif ( isFD == 2 ) then
				frame.Status:SetText("SoR");
				frame.status = "spiritofredemption";
			end

		elseif ( stats and stats["Ressed"] ) then
			frame.HPBar:Hide();
			frame.HPBG:Hide();
			frame.Percent:Hide();
			frame.MPBar:Hide();
			frame.MPBG:Hide();
			frame.Status:Show();
			if ( stats["Ressed"] == 1 ) then
				frame.Status:SetText("Resurrected");
			elseif ( stats["Ressed"] == 2 ) then
				frame.Status:SetText("SS Available");
			else
				frame.Status:SetText("Resurrected");
			end
			frame.status = "resurrected";

		elseif ( isDead ) then
			frame.HPBar:Hide();
			frame.HPBG:Hide();
			frame.Percent:Hide();
			frame.MPBar:Hide();
			frame.MPBG:Hide();
			frame.Status:Show();
			frame.Status:SetText("DEAD");
			frame.status = "dead";

		elseif ( stats and stats["AFK"] and tempOptions["ShowAFK"] ) then
			frame.HPBar:Hide();
			frame.HPBG:Hide();
			frame.Percent:Hide();
			frame.MPBar:Hide();
			frame.MPBG:Hide();
			frame.Status:Show();
			frame.Status:SetText("AFK");
			frame.status = "afk";

		elseif ( health and healthmax and not UnitIsDead(raidid) and not UnitIsGhost(raidid) ) then
			-- && The following code was copied from the CT_RA_UpdateUnitHealth() function and modified:
			local showHP = tempOptions["ShowHP"];
			local memberHeight = 40;  -- && tempOptions["MemberHeight"];
			local maxHealth = healthmax;
			local percent = floor(health / maxHealth * 100);

			frame.HPBar:SetMinMaxValues(0, healthmax);
			frame.HPBar:SetValue(health);
			frame.Percent:Show();

			frame.status = nil;

			if ( showHP and showHP == 1 and maxHealth and memberHeight == 40 ) then
				frame.Percent:SetText(floor(percent/100*maxHealth) .. "/" .. maxHealth);
			elseif ( showHP and showHP == 2 and memberHeight == 40 ) then
				frame.Percent:SetText(percent .. "%");
			elseif ( showHP and showHP == 3 and memberHeight == 40 ) then
				if ( maxHealth ) then
					local diff = floor(percent/100*maxHealth)-maxHealth;
					if ( diff == 0 ) then diff = ""; end
					frame.Percent:SetText(diff);
				else
					frame.Percent:SetText(percent-100 .. "%");
				end
			else
				frame.Percent:Hide();
			end
			local hppercent = percent/100;
			if ( hppercent >= 0 and hppercent <= 1 ) then
				local r, g;
				if ( hppercent > 0.5 and hppercent <= 1) then
					g = 1;
					r = (1.0 - hppercent) * 2;
				elseif ( hppercent >= 0 and hppercent <= 0.5 ) then
					r = 1.0;
					g = hppercent * 2;
				else
					r = 0;
					g = 1;
				end
				frame.HPBar:SetStatusBarColor(r, g, 0);
				frame.HPBG:SetVertexColor(r, g, 0, tempOptions["BGOpacity"]);
			end
		else
			frame.status = nil;

			frame.HPBar:Hide();
			frame.HPBG:Hide();
		end
		frame.Name:SetText(EkCTRA_MTTargets_UnitName(raidid));
-- &&		if ( UnitCanAttack("player", raidid) ) then
-- &&			frame.Name:SetTextColor(1, 0.5, 0);
-- &&		else
			frame.Name:SetTextColor(0.5, 1, 0);
-- &&		end
		frame.unitName = EkCTRA_MTTargets_UnitName(raidid);
	else
		frame.Percent:Hide();
		frame.HPBar:Hide();
		frame.HPBG:Hide();
		frame.MPBar:Hide();
		frame.MPBG:Hide();
		frame.Status:Hide();
		if (isPet) then
			frame.Name:SetText(val .. "'s Pet");
		else
			frame.Name:SetText(val);  -- &&   .. "'s Target");
		end
		frame.Name:SetHeight(30);
		frame.Name:SetTextColor(1, 0.82, 0);
		frame:SetBackdropColor(0.3, 0.3, 0.3, 1);
	end

	if (frame.status) then
		frame:SetBackdropColor(0.3, 0.3, 0.3, 1);
	end
end


-- ------------------------------------------------------------
-- Hooked function: CT_RA_UpdateRaidFrames (from CT_RaidAssist.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RA_UpdateRaidFrames()
	-- ----------
	-- This function gets called instead of CT_RA_UpdateRaidFrames().
	-- ----------

	-- Call the original function first
	if (EkCTRA_MTTargets_OldFunc.CT_RA_UpdateRaidFrames) then
		EkCTRA_MTTargets_OldFunc.CT_RA_UpdateRaidFrames();
	end

	if (EkCTRA_MTTargets_Status == 0) then
		return;
	end



	-- && Added code to fix a bug in the CT_RaidAssist function.
	-- It was not hiding/showing the MT group name if you toggled
	-- the "Display Main Tanks" checkbox.
if (not InCombatLockdown()) then
	local tempOptions = CT_RAMenu_Options["temp"];
	local numRaidMembers = CT_RA_NumRaidMembers;
	
	local showGroups = tempOptions["ShowGroups"];
	local lockGroups = tempOptions["LockGroups"];
	local hideNames = tempOptions["HideNames"];

	-- Main Tanks
-- && Added the "not tempOptions["HideMTs"] and" test.
	if ( not tempOptions["HideMTs"] and numRaidMembers > 0 and next(CT_RA_MainTanks) ) then
		CT_RAMTGroup:Show();
		if ( hideNames ) then
			CT_RAMTGroup.GroupName:Hide();
		else
			CT_RAMTGroup.GroupName:Show();
		end
		if ( lockGroups ) then
			CT_RAMTGroupDrag:Hide();
		else
			CT_RAMTGroupDrag:Show();
		end
	else
		CT_RAMTGroup:Hide();
		CT_RAMTGroupDrag:Hide();
		CT_RAMTGroup.GroupName:Hide(); -- && Added this line.
	end
end


	if (EkCTRA_MTTargets_Status == 1) then
		-- Handle the Main Tank Players and Pets
		EkCTRA_MTTargets_EK_RA_UpdateRaidFrames()
	end
end

function EkCTRA_MTTargets_EK_RA_UpdateRaidFrames()
	-- ----------
	-- This is a modified copy of CT_RA_UpdateRaidFrames().
	-- ----------
	if (InCombatLockdown()) then
		return;
	end

	local showmts, showmtplayers, showmtpets = EkCTRA_MTTargets_GetShowStatus();

	local tempOptions = CT_RAMenu_Options["temp"];
	local numRaidMembers = CT_RA_NumRaidMembers;
	
	local showGroups = tempOptions["ShowGroups"];
	local lockGroups = tempOptions["LockGroups"];
	local hideNames = tempOptions["HideNames"];
	

	if (showmtplayers) then
		-- Main Tank Players
		if ( numRaidMembers > 0 and next(CT_RA_MainTanks) ) then
			CT_RAMTGroupPlayer:Show();

			if ( hideNames ) then
				CT_RAMTGroupPlayer.GroupName:Hide();
			else
				CT_RAMTGroupPlayer.GroupName:Show();
			end

			if ( lockGroups ) then
				CT_RAMTGroupPlayerDrag:Hide();
			else
				CT_RAMTGroupPlayerDrag:Show();
			end
		else
			CT_RAMTGroupPlayer:Hide();
			CT_RAMTGroupPlayerDrag:Hide();
		end
	else
		CT_RAMTGroupPlayer:Hide();
		CT_RAMTGroupPlayerDrag:Hide();
	end

	if (showmtpets) then
		-- Main Tank Pets
		if ( numRaidMembers > 0 and next(CT_RA_MainTanks) ) then
			local foundPet;

			CT_RAMTGroupPet:Show();

			if (EkCTRA_MTTargets_PlayerHasNoPet == 1) then
				-- Hide pet boxes for players that don't have a pet.
				local tempOptions = CT_RAMenu_Options["temp"];
				local num, val = 0;
				for i = 1, ( tempOptions["ShowNumMTs"] or 10 ), 1 do
					val = CT_RA_MainTanks[i];
					if ( val ) then
						num = num + 1;
						local frame = getglobal("CT_RAMTGroupPetUnitButton"..num);
						if ( frame ) then
							local raidid = SecureButton_GetUnit(frame); -- frame.unit;
							if ( raidid ) then
								if (UnitExists(raidid)) then
									foundPet = 1;
									frame:Show();
								else
									frame:Hide();
								end
							else
								frame:Hide();
							end
						end
					end
				end
			end

			if ( hideNames ) then
				CT_RAMTGroupPet.GroupName:Hide();
			else
				CT_RAMTGroupPet.GroupName:Show();
			end

			if ( lockGroups ) then
				CT_RAMTGroupPetDrag:Hide();
			else
				CT_RAMTGroupPetDrag:Show();
			end
		else
			CT_RAMTGroupPet:Hide();
			CT_RAMTGroupPetDrag:Hide();
		end
	else
		CT_RAMTGroupPet:Hide();
		CT_RAMTGroupPetDrag:Hide();
	end


end


-- ------------------------------------------------------------
-- Modified copy of function: prepareFrame (from CT_RaidAssist.lua)
-- ------------------------------------------------------------

-- Locally bound frame cache
local frameCache = CT_RA_Cache;

local function prepareFrame(frame, dragFrame, template, initFunction, title, splitView, useModifier, showmts, showmtplayers, showmtpets)
	local tempOptions = CT_RAMenu_Options["temp"];
	local numRaidMembers = CT_RA_NumRaidMembers;
	local initialConfigFunction = initFunction;

	local showReversed = tempOptions["ShowReversed"];
	local showHorizontal = splitView == nil and tempOptions["ShowHorizontal"];
	local hideBorder = tempOptions["HideBorder"];
	local removeSpace = hideBorder and tempOptions["HideSpace"];
	
	if ( splitView ~= 1 and not frame.init ) then
		frame:SetAttribute("template", template);
		frame.initialConfigFunction = initialConfigFunction;
		frame.GroupName = frameCache[frame].GroupName;
		frame.id = frame:GetID();
		frame.name = frame:GetName();
		frame.frameParent = frame:GetParent();
		frame.GroupName:ClearAllPoints();
		frame.GroupName:SetPoint("CENTER", dragFrame);
		frame.init = true;
		frame.useModifier = useModifier;
		if ( title ) then
			frame.GroupName:SetText(title);
		end
	end
	
--	local splitOffset = ( hideBorder and 5 ) or 0;

	local point1;
	if ( showReversed ) then
		point1 = "BOTTOM";
	else
		point1 = "TOP";
		if (splitView ~= 1) then
			frame:SetAttribute("sortDir", "ASC");
		end
	end

	local attach, relFrame, offset, ancGrp = prepareFrame2(splitView, showmts, showmtplayers, showmtpets);

	frame:SetAttribute("point", point1);

	if (not (EkCTRA_MTTargets_AttachedTo or EkCTRA_MTTargets_DragAll or dragFrame.ekdragMode)) then
		if (attach) then
			if (splitView ~= ancGrp) then
				EkCTRA_MTTargets_SavePoint[splitView] = { dragFrame, relFrame, point1, offset, 0 };
			end

			offset = offset * CT_RAMenu_Options["temp"]["MTScaling"];

			-- Anchor one group to the other using their drag frames.
			dragFrame:ClearAllPoints();
			dragFrame:SetPoint("CENTER", relFrame, "CENTER", offset, 0);

			-- Re-anchor the drag frame relative to the UIParent.
			local t = dragFrame:GetTop();
			local l = dragFrame:GetLeft();
			if (t and l) then
				dragFrame:ClearAllPoints();
				dragFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", l, t);
			end
		else
			-- Re-anchor the drag frame relative to the UIParent.
			local t = dragFrame:GetTop();
			local l = dragFrame:GetLeft();
			if (t and l) then
				dragFrame:ClearAllPoints();
				dragFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", l, t);
			end
		end
	end

	if (splitView == 1) then
		return;
	end

	if ( removeSpace ) then
		if ( showHorizontal ) then
			frame:SetAttribute("xOffset", 10);
			frame:SetAttribute("yOffset", 0);
		else
			frame:SetAttribute("xOffset", 0);
			frame:SetAttribute("yOffset", ( showReversed and -10 ) or 10);
		end
	else
		frame:SetAttribute("xOffset", 0);
		frame:SetAttribute("yOffset", 0);
	end
	return frame;
end


function prepareFrame2(splitView, showmts, showmtplayers, showmtpets)
	local ord, pos, grp, grpDis, grpVis;
	local ancPos, ancGrp, ancDrag;

	local mtwidth, ptwidth, height = CT_RA_GetMTFrameData();

	local tempOptions = CT_RAMenu_Options["temp"];
	local showMTT = tempOptions["ShowMTTT"] or false;
	local hideBorder = tempOptions["HideBorder"];

	-- Positions of the groups if they are all shown.
	-- Position 1 == left, 2 == middle, 3 == right.
	-- Group 1 == Target, 2 == Player, 3 == Pet.
	-- ord == Array of group numbers (subscript == position, value == group number).
	-- pos == Array of positions (subscript == group, value == position).
	if (EkCTRA_MTTargets_SidePlayer == 0) then
		if (EkCTRA_MTTargets_SidePet == 0) then
			if (EkCTRA_MTTargets_SidePetPlayer == 0) then
				ord = {3,2,1}; -- Pet, Player, Target
				pos = {3,2,1};
			else
				ord = {2,3,1}; -- Player, Pet, Target
				pos = {3,1,2};
			end
		else
			ord = {2,1,3}; -- Player, Target, Pet
			pos = {2,1,3};
		end
	else
		if (EkCTRA_MTTargets_SidePet == 1) then
			if (EkCTRA_MTTargets_SidePetPlayer == 0) then
				ord = {1,3,2}; -- Target, Pet, Player
				pos = {1,3,2};
			else
				ord = {1,2,3}; -- Target, Player, Pet
				pos = {1,2,3};
			end
		else
			ord = {3,1,2}; -- Pet, Target, Player
			pos = {2,3,1};
		end
	end

	-- Determine which group the others will anchor to.
	if (EkCTRA_MTTargets_AnchorToGroup) then
		local ancTo = EkCTRA_MTTargets_AnchorToGroup;
		if (ancTo == 1 and showmts) then
			ancGrp = 1;
		elseif (ancTo == 2 and showmtplayers) then
			ancGrp = 2;
		elseif (ancTo == 3 and showmtpets) then
			ancGrp = 3;
		end
	end

	if (not ancGrp) then
		if (showmts) then
			-- Anchor to the Targets group
			ancGrp = 1;
		elseif (showmtplayers) then
			-- Anchor to the Players group
			ancGrp = 2;
		elseif (showmtpets) then
			-- Anchor to the Pets group
			ancGrp = 3;
		else
			-- Nothing visible, so anchor to the Players group.
			ancGrp = 2;
		end
	end

	ancDrag = getglobal(EkCTRA_MTTargets_Groups[ancGrp] .. "Drag");

	-- Establish visibility of each group.
	grpVis = { showmts, showmtplayers, showmtpets };

	-- Pretend the anchor group is visible (for the purpose of determining distances from the anchor).
	-- The ony time the anchor group won't be visible, is when all of the groups are not visible.
	grpVis[ancGrp] = 1;

	-- Determine the position (1,2,3) of the anchor group.
	ancPos = pos[ancGrp];

	-- Determine the distance of each group to the anchor.
	-- The anchor will have a distance value of 0.
	-- 1 frame to the left of the anchor is a distance value of -1, etc.
	-- 1 frame to the right of the anchor is a distance value of 1, etc.
	grpDis = {};
	grpDis[ancGrp] = 0;
	local d = 0;
	for p = ancPos - 1, 1, -1 do
		-- If the group in the previous position (ie. to the right) was visible, then change the value of d.
		-- If it wasn't visible, then we want this group to sit on top of the previous one.
		if (grpVis[ord[p+1]]) then
			d = d - 1;
		end
		-- Save the distance from the anchor to this group.
		grpDis[ord[p]] = d;
	end
	d = 0;
	for p = ancPos + 1, 3, 1 do
		-- If the group in the previous position (ie. to the left) was visible, then change the value of d.
		-- If it wasn't visible, then we want this group to sit on top of the previous one.
		if (grpVis[ord[p-1]]) then
			d = d + 1;
		end
		-- Save the distance from the anchor to this group.
		grpDis[ord[p]] = d;
	end


	local grp, attach, relFrame, offset;

	-- Set which group we are using.
	grp = splitView;

	if (EkCTRA_MTTargets_LockJoined == 1 and grp ~= ancGrp) then
		local width = mtwidth;
		if (hideBorder) then
			if (EkCTRA_MTTargets_GapHide == 1) then
				width = width - 10;
			end
		end
		offset = width * grpDis[grp];
		if (grp == 1 or ancGrp == 1) then
			-- Anchoring the Targets group to a non-Targets group, or vice versa.
			-- If Targets Targets are visible, then we need to further adjust the offset by half of the frame width.
			if (showMTT) then
				local adjust = width / 2;
				if (offset < 0) then
					offset = offset - adjust;
				else
					offset = offset + adjust;
				end
			end
		end

		relFrame = ancDrag;
		attach = 1;
	end

	return attach, relFrame, offset, ancGrp;
end


-- ------------------------------------------------------------
-- Hooked function: CT_RA_UpdateRaidFrameData (from CT_RaidAssist.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RA_UpdateRaidFrameData()
	-- ----------
	-- This function gets called instead of CT_RA_UpdateRaidFrameData().
	-- ----------

	if (EkCTRA_MTTargets_Status == 1) then
		-- Handle the Main Tank Players and Pets
		EkCTRA_MTTargets_EK_RA_UpdateRaidFrameData()
	end

	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RA_UpdateRaidFrameData) then
		EkCTRA_MTTargets_OldFunc.CT_RA_UpdateRaidFrameData();
	end

end

function EkCTRA_MTTargets_EK_RA_UpdateRaidFrameData()
	-- ----------
	-- This is a modified copy of CT_RA_UpdateRaidFrameData().
	-- ----------
	if (InCombatLockdown()) then
		return;
	end

	local showmts, showmtplayers, showmtpets = EkCTRA_MTTargets_GetShowStatus();

	local tempOptions = CT_RAMenu_Options["temp"];
	local numRaidMembers = CT_RA_NumRaidMembers;

	-- Main Tanks
	local showMTT = tempOptions["ShowMTTT"] or false;

	local list, num, obj = "", 0;
-- &&	if ( not tempOptions["HideMTs"] ) then
	if ( showmts or showmtplayers or showmtpets ) then
		for i = 1, 10, 1 do
			obj = CT_RA_MainTanks[i];
			if ( obj ) then
				list = list .. obj .. ",";
				num = num + 1;
				if ( num == tempOptions["ShowNumMTs"] ) then
					break;
				end
			end
		end
		list = strsub(list, 0, -2);
	end

	EkCTRA_MTTargets_SavePoint = {};

--	if (showmtplayers) then
		local frame = prepareFrame(CT_RAMTGroupPlayer, CT_RAMTGroupPlayerDrag, "CT_RAMTMemberTemplate", CT_RA_SetupMTFrame, "MT Players", 2, "", showmts, showmtplayers, showmtpets);
		frame:SetAttribute("nameList", list);
--	end

--	if (showmtpets) then
		local frame = prepareFrame(CT_RAMTGroupPet, CT_RAMTGroupPetDrag, "CT_RAMTMemberTemplate", CT_RA_SetupMTFrame, "MT Pets", 3, "pet", showmts, showmtplayers, showmtpets);
		frame:SetAttribute("nameList", list);
--	end

--	if (not showmts) then
		-- This prepareFrame call is just to position the MT Targets drag frame. Populating it with names will be done
		-- by the original function, which gets called when we return from here.
		-- We're passing a 1 as the splitView value, rather than the -1 or 1 that the original CT_RaidAssist code passed.
		prepareFrame(CT_RAMTGroup, CT_RAMTGroupDrag, "CT_RAMTMemberTemplate", CT_RA_SetupMTFrame, "MT Targets", 1, "target", showmts, showmtplayers, showmtpets);
--	end

	EkCTRA_MTTargets_AnchorToGroup = nil;
end


-- ------------------------------------------------------------
-- Hooked function: CT_RA_UpdateRaidFrameOptions (from CT_RaidAssist.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RA_UpdateRaidFrameOptions()
	-- ----------
	-- This function gets called instead of CT_RA_UpdateRaidFrameOptions().
	-- ----------

	if (EkCTRA_MTTargets_Status == 1) then
		-- Handle the Main Tank Players and Pets
		EkCTRA_MTTargets_EK_RA_UpdateRaidFrameOptions()
	end

	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RA_UpdateRaidFrameOptions) then
		EkCTRA_MTTargets_OldFunc.CT_RA_UpdateRaidFrameOptions();
	end
end

function EkCTRA_MTTargets_EK_RA_UpdateRaidFrameOptions()
	-- ----------
	-- This is a modified copy of CT_RA_UpdateRaidFrameOptions().
	-- ----------
	if (InCombatLockdown()) then
		return;
	end

	local showmts, showmtplayers, showmtpets = EkCTRA_MTTargets_GetShowStatus();

	local tempOptions = CT_RAMenu_Options["temp"];
	local numRaidMembers = CT_RA_NumRaidMembers;

	-- Main Tank Players
	local showMTT = tempOptions["ShowMTTT"] or false;

	local list, num, obj = "", 0;
-- &&	if ( not tempOptions["HideMTs"] ) then
	if ( showmts or showmtplayers or showmtpets ) then
		for i = 1, 10, 1 do
			obj = CT_RA_MainTanks[i];
			if ( obj ) then
				list = list .. obj .. ",";
				num = num + 1;
				if ( num == tempOptions["ShowNumMTs"] ) then
					break;
				end
			end
		end
		list = strsub(list, 0, -2);
	end

	EkCTRA_MTTargets_SavePoint = {};

--	if (showmtplayers) then
		local frame = prepareFrame(CT_RAMTGroupPlayer, CT_RAMTGroupPlayerDrag, "CT_RAMTMemberTemplate", CT_RA_SetupMTFrame, "MT Players", 2, "", showmts, showmtplayers, showmtpets);
		frame:SetAttribute("nameList", list);
--	end

--	if (showmtpets) then
		local frame = prepareFrame(CT_RAMTGroupPet, CT_RAMTGroupPetDrag, "CT_RAMTMemberTemplate", CT_RA_SetupMTFrame, "MT Pets", 3, "pet", showmts, showmtplayers, showmtpets);
		frame:SetAttribute("nameList", list);
--	end

--	if (not showmts) then
		-- This prepareFrame call is just to position the MT Targets drag frame. Populating it with names will be done
		-- by the original function, which gets called when we return from here.
		-- We're passing a 1 as the splitView value, rather than the -1 or 1 that the original CT_RaidAssist code passed.
		prepareFrame(CT_RAMTGroup, CT_RAMTGroupDrag, "CT_RAMTMemberTemplate", CT_RA_SetupMTFrame, "MT Targets", 1, "target", showmts, showmtplayers, showmtpets);
--	end

	EkCTRA_MTTargets_AnchorToGroup = nil;
end


-- ------------------------------------------------------------
-- Hooked function: CT_RAMenuMisc_OnUpdate (from CT_RAMenu.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RAMenuMisc_OnUpdate(elapsed)
	-- ----------
	-- This function gets called instead of CT_RAMenuMisc_OnUpdate().
	-- ----------

	if (EkCTRA_MTTargets_Status == 0) then
		if (EkCTRA_MTTargets_OldFunc.CT_RAMenuMisc_OnUpdate) then
			EkCTRA_MTTargets_OldFunc.CT_RAMenuMisc_OnUpdate(elapsed);
		end
		return;
	end

	-- Handle the Main Tank Players and Pets
	EkCTRA_MTTargets_EK_RAMenuMisc_OnUpdate(elapsed)
end

function EkCTRA_MTTargets_EK_RAMenuMisc_OnUpdate(elapsed)
	-- ----------
	-- This is a modified copy of CT_RA_UpdateRaidFrameOptions().
	-- ----------
	local tempOptions = CT_RAMenu_Options["temp"];
	local newScaling = CT_RAMenu_Options["temp"]["MTScaling"];

	local scaleupdate;
	if ( this.scaleupdate ) then

		local temp = this.scaleupdate - elapsed;
		if ( temp <= 0 ) then

			scaleupdate = true;
			if ( CT_RAMenu_Options["temp"]["MTScaling"] ) then
--				local newScaling = CT_RAMenu_Options["temp"]["MTScaling"];
				local mtwidth, ptwidth, height = CT_RA_GetMTFrameData();
-- &&
--[[
				CT_RAMTGroup:SetScale(newScaling);
				CT_RAMTTGroup:SetScale(newScaling);
				CT_RAPTGroup:SetScale(newScaling);
				CT_RAPTTGroup:SetScale(newScaling);
				CT_RAMTGroupDrag:SetWidth(mtwidth*newScaling+(22*newScaling));
				CT_RAMTGroupDrag:SetHeight(height*newScaling/2);
				CT_RAPTGroupDrag:SetWidth(ptwidth*newScaling+(22*newScaling));
				CT_RAPTGroupDrag:SetHeight(height*newScaling/2);
]]

				if (not InCombatLockdown()) then
					CT_RAMTGroupPlayer:SetScale(newScaling);
					CT_RAMTGroupPet:SetScale(newScaling);

					CT_RAMTGroupPlayerDrag:SetWidth(mtwidth*newScaling); -- +(22*newScaling));
					CT_RAMTGroupPlayerDrag:SetHeight(height*newScaling/2);
					CT_RAMTGroupPlayerMenu:SetWidth(mtwidth*newScaling); -- +(22*newScaling));
					CT_RAMTGroupPlayerMenu:SetHeight(height*newScaling/2);

					CT_RAMTGroupPetDrag:SetWidth(mtwidth*newScaling); -- +(22*newScaling));
					CT_RAMTGroupPetDrag:SetHeight(height*newScaling/2);
					CT_RAMTGroupPetMenu:SetWidth(mtwidth*newScaling); -- +(22*newScaling));
					CT_RAMTGroupPetMenu:SetHeight(height*newScaling/2);
				end
			end

		end
	end

	if (EkCTRA_MTTargets_OldFunc.CT_RAMenuMisc_OnUpdate) then
		EkCTRA_MTTargets_OldFunc.CT_RAMenuMisc_OnUpdate(elapsed);
	end

	CT_RAMTGroupMenu:SetWidth(CT_RAMTGroupDrag:GetWidth());
	CT_RAMTGroupMenu:SetHeight(CT_RAMTGroupDrag:GetHeight());

	if (InCombatLockdown()) then
		return;
	end

	if (not (EkCTRA_MTTargets_AttachedTo or EkCTRA_MTTargets_DragAll)) then
		for k, v in ipairs(EkCTRA_MTTargets_SavePoint) do
			local drag = v[1];
			if (not drag.ekdragMode) then
				drag:ClearAllPoints();
				drag:SetPoint(v[3], v[2], v[3], v[4] * newScaling, v[5]);

				-- Re-anchor the drag frame relative to the UIParent.
				local t = drag:GetTop();
				local l = drag:GetLeft();
				if (t and l) then
					drag:ClearAllPoints();
					drag:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", l, t);
				end
			end
		end
	end
end


-- ------------------------------------------------------------
-- Hooked function: CT_RAMenuAdditional_ScalingMT_OnValueChanged (from CT_RAMenu.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RAMenuAdditional_ScalingMT_OnValueChanged()
	-- ----------
	-- This function gets called instead of CT_RA_UpdateRaidFrameOptions().
	-- ----------

	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RAMenuAdditional_ScalingMT_OnValueChanged) then
		EkCTRA_MTTargets_OldFunc.CT_RAMenuAdditional_ScalingMT_OnValueChanged();
	end

	if (EkCTRA_MTTargets_Status == 0) then
		return;
	end

	-- Handle the Main Tank Players and Pets
	EkCTRA_MTTargets_EK_RAMenuAdditional_ScalingMT_OnValueChanged()
end

function EkCTRA_MTTargets_EK_RAMenuAdditional_ScalingMT_OnValueChanged()
	-- ----------
	-- This is a modified copy of CT_RAMenuAdditional_ScalingMT_OnValueChanged().
	-- ----------
	local tempOptions = CT_RAMenu_Options["temp"];

	local newScaling = CT_RAMenu_Options["temp"]["MTScaling"];
	local mtwidth, ptwidth, height = CT_RA_GetMTFrameData();

--[[
	CT_RAMTGroup:SetScale(newScaling);
	CT_RAMTTGroup:SetScale(newScaling);
	CT_RAPTGroup:SetScale(newScaling);
	CT_RAPTTGroup:SetScale(newScaling);
	CT_RAMTGroupDrag:SetWidth(mtwidth*newScaling+(22*newScaling));
	CT_RAMTGroupDrag:SetHeight(height*newScaling/2);
	CT_RAPTGroupDrag:SetWidth(ptwidth*newScaling+(22*newScaling));
	CT_RAPTGroupDrag:SetHeight(height*newScaling/2);
]]


	if (not InCombatLockdown()) then
		CT_RAMTGroupPlayer:SetScale(newScaling);
		CT_RAMTGroupPet:SetScale(newScaling);

		CT_RAMTGroupPlayerDrag:SetWidth(mtwidth*newScaling); -- +(22*newScaling));
		CT_RAMTGroupPlayerDrag:SetHeight(height*newScaling/2);
		CT_RAMTGroupPlayerMenu:SetWidth(mtwidth*newScaling); -- +(22*newScaling));
		CT_RAMTGroupPlayerMenu:SetHeight(height*newScaling/2);

		CT_RAMTGroupPetDrag:SetWidth(mtwidth*newScaling); -- +(22*newScaling));
		CT_RAMTGroupPetDrag:SetHeight(height*newScaling/2);
		CT_RAMTGroupPetMenu:SetWidth(mtwidth*newScaling); -- +(22*newScaling));
		CT_RAMTGroupPetMenu:SetHeight(height*newScaling/2);
	end

	CT_RAMTGroupMenu:SetWidth(CT_RAMTGroupDrag:GetWidth());
	CT_RAMTGroupMenu:SetHeight(CT_RAMTGroupDrag:GetHeight());

	if (InCombatLockdown()) then
		return;
	end

	if (not (EkCTRA_MTTargets_AttachedTo or EkCTRA_MTTargets_DragAll)) then
		for k, v in ipairs(EkCTRA_MTTargets_SavePoint) do
			local drag = v[1];
			if (not drag.ekdragMode) then
				drag:ClearAllPoints();
				drag:SetPoint(v[3], v[2], v[3], v[4] * newScaling, v[5]);

				-- Re-anchor the drag frame relative to the UIParent.
				local t = drag:GetTop();
				local l = drag:GetLeft();
				if (t and l) then
					drag:ClearAllPoints();
					drag:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", l, t);
				end
			end
		end
	end
end


-- ------------------------------------------------------------
-- Hooked script: OnMouseDown for the drag frames (from CT_RaidAssist.xml)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RAMTGroupDragOnMouseDown()
	-- ----------
	-- Mouse button pressed on a drag frame.
	--
	-- Called instead of the OnMouseDown </Script> code for CT_RAMTGroupDrag
	-- which is inherited from CT_RAGroupDragTemplate.
	--
	-- The original function is called at the end if the
	-- event is still unhandled.
	-- ----------
	local mtdrag, mtmenu;
	local cFrame;

	if (EkCTRA_MTTargets_Status == 0) then
		-- Call original function.
		if (EkCTRA_MTTargets_OldFunc.CT_RAMTGroupDragOnMouseDown) then
			EkCTRA_MTTargets_OldFunc.CT_RAMTGroupDragOnMouseDown();
		end
		return;
	end

	cFrame = this:GetName();

	if (cFrame == "CT_RAMTGroupPlayerMenu") then
		mtmenu = 1;
		EkCTRA_MTTargets_MenuFrameNum = 2;
	elseif (cFrame == "CT_RAMTGroupPetMenu") then
		mtmenu = 1;
		EkCTRA_MTTargets_MenuFrameNum = 3;
	elseif (cFrame == "CT_RAMTGroupMenu") then
		mtmenu = 1;
		EkCTRA_MTTargets_MenuFrameNum = 1;
	else
		cFrame = this:GetName();
		for key, val in ipairs(EkCTRA_MTTargets_Groups) do
			if (cFrame == val .. "Drag") then
				mtdrag = 1;
				EkCTRA_MTTargets_MenuFrameNum = key;
				break;
			end
		end
	end

	if (arg1 == "LeftButton") then
		if (mtdrag) then
			local dragMode;

			if (EkCTRA_MTTargets_AttachedTo) then
				-- The groups are attached together, which means that
				-- the user wants to keep them joined, and we are in
				-- combat lockdown (or we are about to be).

				-- Only allow the user to drag the "attached to" group.
				if (EkCTRA_MTTargets_AttachedTo[1]:GetName() == cFrame) then
					dragMode = 2;
				else
					-- Don't allow this drag frame to be dragged.
					return;
				end

			-- The groups are not attached together. Either the user wants
			-- to keep them unjoined, or we are not in combat lockdown.

			elseif (EkCTRA_MTTargets_LockJoined == 1) then
				dragMode = 2;   -- Drag all MT windows at the same time.

			elseif (IsShiftKeyDown()) then
				if (InCombatLockdown()) then
					dragMode = 1;   -- Drag individual group.
				else
					dragMode = 2;   -- Drag all MT windows at the same time.
				end
			else
				dragMode = 1;   -- Drag individual group.
			end

			if (dragMode) then
				this.ekdragMode = dragMode;

				if (dragMode == 2) then
					EkCTRA_MTTargets_DragAll = 1;
					if (not EkCTRA_MTTargets_AttachedTo) then
						-- We are not currently in combat lockdown (and we are not about to be).
						EkCTRA_MTTargets_AttachAllMTWindows(this);
					end
					this:StartMoving();
					return;

				elseif (dragMode == 1) then
					this:StartMoving();
					return;
				end
			end
		end

	elseif (arg1 == "RightButton") then
		if (mtdrag or mtmenu) then

			if (EkCTRA_MTTargets_DragAll or frame.ekdragMode) then
				-- Don't open menu while user is dragging the frame.
				return;
			end

			local oDrop = getglobal("EkCTRA_MTTargets_DropDown");
			local cFrame = this:GetName();
			ToggleDropDownMenu(1, nil, oDrop, cFrame, 47, 15);
			return;
		end
	end

	if (mtmenu) then
		return;
	end

	-- Call original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RAMTGroupDragOnMouseDown) then
		EkCTRA_MTTargets_OldFunc.CT_RAMTGroupDragOnMouseDown();
	end
end



-- ------------------------------------------------------------
-- Hooked script: OnMouseUp for the drag frames (from CT_RaidAssist.xml)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RAMTGroupDragOnMouseUp()
	-- ----------
	-- Mouse button released on a drag frame.
	--
	-- Called instead of the OnMouseUp </Script> code for CT_RAMTGroupDrag
	-- which is inherited from CT_RAGroupDragTemplate.
	--
	-- The original function is called at the end if the
	-- event is still unhandled.
	-- ----------
	local mtdrag;
	local cFrame;

	if (EkCTRA_MTTargets_Status == 0) then
		-- Call original function.
		if (EkCTRA_MTTargets_OldFunc.CT_RAMTGroupDragOnMouseUp) then
			EkCTRA_MTTargets_OldFunc.CT_RAMTGroupDragOnMouseUp();
		end
		return;
	end

	cFrame = this:GetName();
	for key, val in ipairs(EkCTRA_MTTargets_Groups) do
		if (cFrame == val .. "Drag") then
			mtdrag = 1;
			break;
		end
	end

	if ( arg1 == "LeftButton" ) then

		if (mtdrag) then

			if ( EkCTRA_MTTargets_AttachedTo ) then
				-- Was dragging all groups.
				this:StopMovingOrSizing();
				this.ekdragMode = nil;
				EkCTRA_MTTargets_DragAll = nil;
				if (not InCombatLockdown()) then
					EkCTRA_MTTargets_UnattachAllMTWindows();
					CT_RA_UpdateRaidFrameData();
					CT_RA_UpdateMTs(true);
				end
				CT_RAMenu_SaveWindowPositions();
				return;
			end

			-- Was dragging a single group.
			this:StopMovingOrSizing();
			this.ekdragMode = nil;
			if (EkCTRA_MTTargets_AlignToTitle == 1) then
				EkCTRA_MTTargets_Update_AlignToTitle();
			end
			CT_RAMenu_SaveWindowPositions();
			return;
		end
	end

	-- Call original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RAMTGroupDragOnMouseUp) then
		EkCTRA_MTTargets_OldFunc.CT_RAMTGroupDragOnMouseUp();
	end
end



-- ------------------------------------------------------------
-- Hooked function: CT_RA_Drag_OnEnter (from CT_RaidAssist.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RA_Drag_OnEnter(oFrame)
	-- ----------
	-- Mouse is over a drag frame.
	--
	-- Called instead of CT_RA_Drag_OnEnter().
	--
	-- Calls the original function first, then
	-- tests for player/pet/target drag frames
	-- in order to customize the tooltip.
	-- ----------
	local mtdrag, mtmenu;
	local cFrame;

	if (not oFrame) then
		oFrame = this;
	end

	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RA_Drag_OnEnter) then
		EkCTRA_MTTargets_OldFunc.CT_RA_Drag_OnEnter();
	end

	if (EkCTRA_MTTargets_Status == 0) then
		return;
	end

	cFrame = oFrame:GetName();

	if (cFrame == "CT_RAMTGroupPlayerMenu") then
		mtmenu = 1;
	elseif (cFrame == "CT_RAMTGroupPetMenu") then
		mtmenu = 1;
	elseif (cFrame == "CT_RAMTGroupMenu") then
		mtmenu = 1;
	else
		for key, val in ipairs(EkCTRA_MTTargets_Groups) do
			if (cFrame == val .. "Drag") then
				mtdrag = 1;
				break;
			end
		end
	end

	if (mtmenu or mtdrag) then
		local xp = "LEFT";
		local yp = "BOTTOM";
		local xo = 0;
		local yo = -15;
		local xthis, ythis = oFrame:GetCenter();
		local xui, yui = UIParent:GetCenter();
		if ( xthis < xui ) then
			xp = "RIGHT";
		end
		if ( ythis < yui ) then
			yp = "TOP";
			yo = 15;
		end
		GameTooltip:SetOwner(oFrame, "ANCHOR_" .. yp .. xp, xo, yo);

--		CT_RAMenuHelp_SetTooltip();

		if (mtdrag) then
			-- MT Players, pets, targets

			-- If the MT groups are attached to each other...
			if (EkCTRA_MTTargets_AttachedTo) then
				-- If we are in combat lockdown mode...
				if (InCombatLockdown()) then
					-- Only allow the user to drag the "attached to" group.
					if (EkCTRA_MTTargets_AttachedTo[1]:GetName() == cFrame) then
						GameTooltip:AddLine(EkCTRA_MTTargets_TEXT_Click_Drag_All);
					end
				else
					-- Not in combat lockdown mode, so user can drag any of the groups.
					GameTooltip:AddLine(EkCTRA_MTTargets_TEXT_Click_Drag_All);
				end
			else
				-- MT groups are not attached together.
				-- If user is keeping the groups joined together...
				if (EkCTRA_MTTargets_LockJoined == 1) then
					-- Show the "click to drag all" line.
					GameTooltip:AddLine(EkCTRA_MTTargets_TEXT_Click_Drag_All);
				else
					-- User is keeping the groups independent.
					GameTooltip:AddLine(EkCTRA_MTTargets_TEXT_Click_Drag);
					if (not InCombatLockdown()) then
						GameTooltip:AddLine(EkCTRA_MTTargets_TEXT_Shift_Click_Drag_All);
					end
				end
			end

			-- If not dragging frame(s), then allow the right click menu.
			if (not (EkCTRA_MTTargets_DragAll or oFrame.ekdragMode)) then
				GameTooltip:AddLine(EkCTRA_MTTargets_TEXT_Right_Click_Menu);
			end
		else
			-- This is not a drag frame. Show just the right click line for opening the menu.
			GameTooltip:AddLine(EkCTRA_MTTargets_TEXT_Right_Click_Menu);
		end

--		GameTooltip:FadeOut();
		GameTooltip:Show();
	end
end


-- ------------------------------------------------------------
-- Drop down menu functions:
-- ------------------------------------------------------------


function EkCTRA_MTTargets_DropDown_OnLoad()
	-- ----------
	-- Initialize drop down menu when it is loaded.
	-- ----------
	UIDropDownMenu_Initialize(this, EkCTRA_MTTargets_DropDown_InitButtons, "MENU");
end


function EkCTRA_MTTargets_DropDown_OnClick()
	-- ----------
	-- User clicked on a drop down menu item.
	-- ----------
	local tempOptions = CT_RAMenu_Options["temp"];

	if ( this.value == "showplayers" ) then
		EkCTRA_MTTargets_Set_Players();

	elseif ( this.value == "hidemtandplayers" ) then
		EkCTRA_MTTargets_Set_HideMTAndPlayers();

	elseif ( this.value == "showplayerbuffs" ) then
		EkCTRA_MTTargets_Set_PlayerBuffs();

	elseif ( this.value == "showpets" ) then
		EkCTRA_MTTargets_Set_Pets();

	elseif ( this.value == "hidemtandpets" ) then
		EkCTRA_MTTargets_Set_HideMTAndPets();

	elseif ( this.value == "lock_joined" ) then
		if (not InCombatLockdown()) then
			if (EkCTRA_MTTargets_LockJoined == 0) then
				EkCTRA_MTTargets_AnchorToGroup = EkCTRA_MTTargets_MenuFrameNum;
			end
			EkCTRA_MTTargets_Set_LockJoined(); -- nil == Toggle
		end

	elseif ( this.value == "side_player" ) then
		EkCTRA_MTTargets_Set_SidePlayer();

	elseif ( this.value == "side_pet" ) then
		EkCTRA_MTTargets_Set_SidePet();

	elseif ( this.value == "side_petplayer" ) then
		EkCTRA_MTTargets_Set_SidePetPlayer();

	elseif ( this.value == "group_has_no_pet" ) then
		EkCTRA_MTTargets_Set_GroupHasNoPet();

	elseif ( this.value == "player_has_no_pet" ) then
		EkCTRA_MTTargets_Set_PlayerHasNoPet();

	elseif ( this.value == "align_to_title" ) then
		EkCTRA_MTTargets_Set_AlignToTitle();

	elseif ( this.value == "toggle_mtt") then
		if (not InCombatLockdown()) then
			-- Toggle MT Targets
			tempOptions["HideMTs"] = not tempOptions["HideMTs"];
			if (CT_RAMenuFrameGeneralMiscShowMTsCB) then
				CT_RAMenuFrameGeneralMiscShowMTsCB:SetChecked(not tempOptions["HideMTs"]);
			end
			CT_RA_UpdateRaidFrameData();
			CT_RA_UpdateMTs(true);
		end

	elseif ( this.value == "toggle_mttt") then
		if (not InCombatLockdown()) then
			-- Toggle MT Targets Targets
			if (tempOptions["ShowMTTT"]) then
				tempOptions["ShowMTTT"] = nil;
			else
				tempOptions["ShowMTTT"] = 1;
			end
			if (CT_RAMenuFrameMiscDisplayShowMTTTCB) then
				CT_RAMenuFrameMiscDisplayShowMTTTCB:SetChecked(tempOptions["ShowMTTT"]);
			end
			CT_RA_UpdateRaidFrameData();
			CT_RA_UpdateMTs(true);
		end

	elseif ( this.value == "toggle_mtrticons" ) then
		EkCTRA_MTTargets_Set_ShowMTRaidTargetIcons();

	elseif ( this.value == "hide_gap" ) then
		EkCTRA_MTTargets_Set_GapHide()

	end
end


function EkCTRA_MTTargets_DropDown_InitButtons()
	-- ----------
	-- Initialize drop down menu buttons.
	-- ----------
	local tempOptions = CT_RAMenu_Options["temp"];
	local dropdown, info;
	local keepShownOnClick = nil;

--	if ( UIDROPDOWNMENU_OPEN_MENU ) then
--		dropdown = getglobal(UIDROPDOWNMENU_OPEN_MENU);
--	else
		dropdown = this;
--	end

	-- x Show Targets group
	-- x    Show Targets Targets group
	-- x    Show raid target icons
	-- x Show Players group
	-- x    Hide when Targets are hidden
	-- x    Show buffs/debuffs
	-- x Show Pets group
	-- x    Hide when Targets are hidden
	-- x    Hide group if there are no pets
	-- x    Hide box if player has no pet
	-- x Join the groups together
	-- x    Show Players to right of Targets
	-- x    Show Pets to right of Targets
	-- x    Show Pets to right of Players
	-- x    No gap when border is hidden
	-- x Align to title after dragging

	info = {};
	info.text = "MT Groups";
	info.isTitle = 1;
	info.justifyH = "CENTER";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = { };
	info.text = EkCTRA_MTTargets_TEXT_Menu_Show_MTTargets;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "toggle_mtt";
	info.notCheckable = nil;
	if (not tempOptions["HideMTs"]) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.keepShownOnClick = keepShownOnClick;
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	info.text = "    " .. EkCTRA_MTTargets_TEXT_Menu_Show_MTTTargets;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "toggle_mttt";
	info.notCheckable = nil;
	if (tempOptions["ShowMTTT"]) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.keepShownOnClick = keepShownOnClick;
	if (tempOptions["HideMTs"]) then
		info.disabled = 1;
	end
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	info.text = "    " .. EkCTRA_MTTargets_TEXT_Menu_Show_MTRaidTargetIcons;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "toggle_mtrticons";
	info.notCheckable = nil;
	if (EkCTRA_MTTargets_ShowMTRaidTargetIcons == 1) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.keepShownOnClick = keepShownOnClick;
	if (tempOptions["HideMTs"]) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	info.text = EkCTRA_MTTargets_TEXT_Menu_Show_MTPlayers;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "showplayers";
	info.notCheckable = nil;
	if ( EkCTRA_MTTargets_MTPlayers == 1 ) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.keepShownOnClick = keepShownOnClick;
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	info.text = "    " .. EkCTRA_MTTargets_TEXT_Menu_Hide_MTAndPlayers;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "hidemtandplayers";
	info.notCheckable = nil;
	if ( EkCTRA_MTTargets_HideMTAndPlayers == 1 ) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.keepShownOnClick = keepShownOnClick;
	if (EkCTRA_MTTargets_MTPlayers ~= 1) then
		info.disabled = 1;
	end
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	info.text = "    " .. EkCTRA_MTTargets_TEXT_Menu_Show_PlayerBuffs;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "showplayerbuffs";
	info.notCheckable = nil;
	if ( EkCTRA_MTTargets_PlayerBuffs == 1 ) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.keepShownOnClick = keepShownOnClick;
	if (EkCTRA_MTTargets_MTPlayers ~= 1) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	info.text = EkCTRA_MTTargets_TEXT_Menu_Show_MTPets;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "showpets";
	info.notCheckable = nil;
	if (EkCTRA_MTTargets_MTPets == 1) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.keepShownOnClick = keepShownOnClick;
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	info.text = "    " .. EkCTRA_MTTargets_TEXT_Menu_Hide_MTAndPets;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "hidemtandpets";
	info.notCheckable = nil;
	if ( EkCTRA_MTTargets_HideMTAndPets == 1 ) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.keepShownOnClick = keepShownOnClick;
	if (EkCTRA_MTTargets_MTPets ~= 1) then
		info.disabled = 1;
	end
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	if (EkCTRA_MTTargets_GroupHasNoPet == 1) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "    " .. EkCTRA_MTTargets_TEXT_Menu_Hide_Group_No_Pets;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "group_has_no_pet";
	info.notCheckable = nil;
	info.keepShownOnClick = keepShownOnClick;
	if (EkCTRA_MTTargets_MTPets ~= 1) then
		info.disabled = 1;
	end
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	if (EkCTRA_MTTargets_PlayerHasNoPet == 1) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "    " .. EkCTRA_MTTargets_TEXT_Menu_Hide_Box_No_Pets;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "player_has_no_pet";
	info.notCheckable = nil;
	info.keepShownOnClick = keepShownOnClick;
	if (EkCTRA_MTTargets_MTPets ~= 1) then
		info.disabled = 1;
	end
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	if (EkCTRA_MTTargets_LockJoined == 1) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = EkCTRA_MTTargets_TEXT_Menu_Join_MTGroups;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "lock_joined";
	info.notCheckable = nil;
	info.keepShownOnClick = keepShownOnClick;
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	if (EkCTRA_MTTargets_SidePlayer == 1) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "    " .. EkCTRA_MTTargets_TEXT_Menu_Side_Player;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "side_player";
	info.notCheckable = nil;
	info.keepShownOnClick = keepShownOnClick;
	if (EkCTRA_MTTargets_LockJoined ~= 1) then
		info.disabled = 1;
	end
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	if (EkCTRA_MTTargets_SidePet == 1) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "    " .. EkCTRA_MTTargets_TEXT_Menu_Side_Pet;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "side_pet";
	info.notCheckable = nil;
	info.keepShownOnClick = keepShownOnClick;
	if (EkCTRA_MTTargets_LockJoined ~= 1) then
		info.disabled = 1;
	end
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	if (EkCTRA_MTTargets_SidePetPlayer == 1) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "    " .. EkCTRA_MTTargets_TEXT_Menu_Side_PetPlayer;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "side_petplayer";
	info.notCheckable = nil;
	info.keepShownOnClick = keepShownOnClick;
	if (EkCTRA_MTTargets_LockJoined ~= 1) then
		info.disabled = 1;
	end
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	if (EkCTRA_MTTargets_GapHide == 1) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = "    " .. EkCTRA_MTTargets_TEXT_Menu_Hide_Gap;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "hide_gap";
	info.notCheckable = nil;
	info.keepShownOnClick = keepShownOnClick;
	if (EkCTRA_MTTargets_LockJoined ~= 1) then
		info.disabled = 1;
	end
	if (InCombatLockdown()) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	if (EkCTRA_MTTargets_AlignToTitle == 1) then
		info.checked = 1;
	else
		info.checked = nil;
	end
	info.text = EkCTRA_MTTargets_TEXT_Menu_Align_To_Title;
	info.func = EkCTRA_MTTargets_DropDown_OnClick;
	info.value = "align_to_title";
	info.notCheckable = nil;
	info.keepShownOnClick = keepShownOnClick;
	if (EkCTRA_MTTargets_LockJoined == 1) then
		info.disabled = 1;
	end
	UIDropDownMenu_AddButton(info);

	info = { };
	info.text = EkCTRA_MTTargets_TEXT_Menu_Close;
	info.func = CloseDropDownMenus;
	info.arg1 = 1;
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);
end


-- ------------------------------------------------------------
-- Hooked function: CT_RA_UpdateRaidMovability (from CT_RaidAssist.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RA_UpdateRaidMovability()
	-- ----------
	-- This function gets called instead of CT_RA_UpdateRaidMovability().
	-- ----------
	local tempOptions = CT_RAMenu_Options["temp"];

	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RA_UpdateRaidMovability) then
		EkCTRA_MTTargets_OldFunc.CT_RA_UpdateRaidMovability();
	end

	if (EkCTRA_MTTargets_Status == 0) then
		return;
	end

	if (InCombatLockdown()) then
		return;
	end

-- && Added this code to fix a bug in the original function. It does not
-- && show/hide the drag frame for the PT Targets.
	if ( tempOptions["LockGroups"]  ) then
		getglobal("CT_RAPTGroupDrag"):Hide();
	else
		for i = 1, 10, 1 do
			if ( CT_RA_MainTanks[i] ) then
				CT_RAPTGroupDrag:Show();
				break;
			else
				CT_RAPTGroupDrag:Hide();
			end
		end
	end



	local showmts, showmtplayers, showmtpets = EkCTRA_MTTargets_GetShowStatus();

	if ( tempOptions["LockGroups"] or (not showmtplayers) ) then
		getglobal("CT_RAMTGroupPlayerDrag"):Hide();
	else
		for i = 1, 10, 1 do
			if ( CT_RA_MainTanks[i] ) then
				CT_RAMTGroupPlayerDrag:Show();
				break;
			else
				CT_RAMTGroupPlayerDrag:Hide();
			end
		end
	end

	if ( tempOptions["LockGroups"] or (not showmtpets) ) then
		getglobal("CT_RAMTGroupPetDrag"):Hide();
	else
		for i = 1, 10, 1 do
			if ( CT_RA_MainTanks[i] ) then
				CT_RAMTGroupPetDrag:Show();
				break;
			else
				CT_RAMTGroupPetDrag:Hide();
			end
		end
	end
end


-- ------------------------------------------------------------
-- Hooked function: CT_RAMenu_General_ResetWindows (from CT_RAMenu.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RAMenu_General_ResetWindows()
	-- ----------
	-- This function gets called instead of CT_RAMenu_General_ResetWindows().
	-- ----------

	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RAMenu_General_ResetWindows) then
		EkCTRA_MTTargets_OldFunc.CT_RAMenu_General_ResetWindows();
	end

	if (EkCTRA_MTTargets_Status == 0) then
		return;
	end

	CT_RAMTGroupPlayerDrag:ClearAllPoints();
	CT_RAMTGroupPetDrag:ClearAllPoints();

	CT_RAMTGroupPlayerDrag:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", 570, -275);
	CT_RAMTGroupPetDrag:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", 475, -275);

	CT_RAMenu_SaveWindowPositions();
end


-- ------------------------------------------------------------
-- Hooked function: CT_RA_UpdateRaidGroupColors (from CT_RaidAssist.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RA_UpdateRaidGroupColors()
	-- ----------
	-- This function gets called instead of CT_RA_UpdateRaidGroupColors().
	-- ----------

	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RA_UpdateRaidGroupColors) then
		EkCTRA_MTTargets_OldFunc.CT_RA_UpdateRaidGroupColors();
	end

	if (EkCTRA_MTTargets_Status == 0) then
		return;
	end

	local tempOptions = CT_RAMenu_Options["temp"];
	local defaultColors = tempOptions["DefaultColor"];
	local r, g, b, a = defaultColors.r, defaultColors.g, defaultColors.b, defaultColors.a;
	for y = 1, 10, 1 do
		local frame = getglobal("CT_RAMTGroupPlayerUnitButton" .. y);
		if (frame) then
			if ( not frame.status ) then
				frame:SetBackdropColor(r, g, b, a);
			end
			frame.Percent:SetTextColor(r, g, b);
		end

		local frame = getglobal("CT_RAMTGroupPetUnitButton" .. y);
		if (frame) then
			frame:SetBackdropColor(r, g, b, a);
			frame.Percent:SetTextColor(r, g, b);
		end
	end
end


-- ------------------------------------------------------------
-- Hooked function: CT_RAMenu_SaveWindowPositions (from CT_RAMenu.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RAMenu_SaveWindowPositions()
	-- ----------
	-- This function gets called instead of CT_RAMenu_SaveWindowPositions().
	-- ----------

	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RAMenu_SaveWindowPositions) then
		EkCTRA_MTTargets_OldFunc.CT_RAMenu_SaveWindowPositions();
	end

	if (EkCTRA_MTTargets_Status == 0) then
		return;
	end

	EkCTRA_MTTargets_WindowPositions = { };
	local left, top, uitop;

	left, top, uitop = CT_RAMTGroupPlayerDrag:GetLeft(), CT_RAMTGroupPlayerDrag:GetTop(), UIParent:GetTop();
	if ( left and top and uitop ) then
		EkCTRA_MTTargets_WindowPositions["CT_RAMTGroupPlayerDrag"] = { left, top-uitop };
	end

	left, top, uitop = CT_RAMTGroupPetDrag:GetLeft(), CT_RAMTGroupPetDrag:GetTop(), UIParent:GetTop();
	if ( left and top and uitop ) then
		EkCTRA_MTTargets_WindowPositions["CT_RAMTGroupPetDrag"] = { left, top-uitop };
	end
end


-- ------------------------------------------------------------
-- Hooked function: CT_RAMenu_UpdateWindowPositions (from CT_RAMenu.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RAMenu_UpdateWindowPositions()
	-- ----------
	-- This function gets called instead of CT_RAMenu_UpdateWindowPositions().
	-- ----------

	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RAMenu_UpdateWindowPositions) then
		EkCTRA_MTTargets_OldFunc.CT_RAMenu_UpdateWindowPositions();
	end

	if (EkCTRA_MTTargets_Status == 0) then
		return;
	end

	if ( EkCTRA_MTTargets_WindowPositions ) then
		for k, v in pairs(EkCTRA_MTTargets_WindowPositions) do
			local oFrame = getglobal(k);
			if (oFrame) then
				oFrame:ClearAllPoints();
				oFrame:SetPoint("TOPLEFT" , "UIParent", "TOPLEFT", v[1], v[2]);
			end
		end
	end
end


-- ------------------------------------------------------------
-- Hooked function: CT_RA_MemberFrame_OnEnter (from CT_RaidAssist.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RA_MemberFrame_OnEnter()
	local tempOptions = CT_RAMenu_Options["temp"];
	local parent = this.frameParent;
	local id = this.id;
	local cFrame = parent.name;

	local oldFunc = true;
	if (EkCTRA_MTTargets_Status == 1) then
		-- Addon is enabled.
		if (strsub(cFrame, 1, 15) == "CT_RAMTGroupPet" ) then
			oldFunc = false;
		end
	end
	if (oldFunc) then
		-- Call the original function.
		if (EkCTRA_MTTargets_OldFunc.CT_RA_MemberFrame_OnEnter) then
			EkCTRA_MTTargets_OldFunc.CT_RA_MemberFrame_OnEnter();
		end
		return;
	end

	local name;
	if ( CT_RA_MainTanks[id] ) then
		name = CT_RA_MainTanks[id];
	end
	for i = 1, GetNumRaidMembers(), 1 do
		local memberName = GetRaidRosterInfo(i);
		if ( name == memberName ) then
			id = i;
			break;
		end
	end

	local unitid = "raidpet"..id;

	if ( SpellIsTargeting() ) then
		SetCursor("CAST_CURSOR");
	end
	if ( SpellIsTargeting() and not SpellCanTargetUnit(unitid) ) then
		SetCursor("CAST_ERROR_CURSOR");
	end
	if ( tempOptions["HideTooltip"] ) then
		return;
	end

	if (not UnitExists(unitid)) then
		return;
	end

	local pname, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(id);

	name = EkCTRA_MTTargets_UnitName(unitid);
	if (not name) then
		name = "";
	end
	if (name == "") then
		name = pname;
	end

	local xp = "LEFT";
	local yp = "BOTTOM";
	local xthis, ythis = this:GetCenter();
	local xui, yui = UIParent:GetCenter();
	if ( xthis < xui ) then
		xp = "RIGHT";
	end
	if ( ythis < yui ) then
		yp = "TOP";
	end
	GameTooltip:SetOwner(this, "ANCHOR_" .. yp .. xp);

	local race, ctype, color, text;

	level = UnitLevel(unitid);
	if (not level) then
		level = 0;
	end
	if (level == 0) then
		level = "?";
	end

	class = UnitClass(unitid);  -- eg. Mage
	if (class) then
		fileName = string.upper(class);  -- eg. MAGE
	else
		fileName = "";
	end

	race = UnitCreatureFamily(unitid);  -- eg. "Imp"
	ctype = UnitCreatureType(unitid);  -- eg. "Demon"
	if (not race) then
		race = ctype;
		ctype = nil;
	end

	color = RAID_CLASS_COLORS[fileName];
	if ( not color ) then
		color = { ["r"] = 1, ["g"] = 1, ["b"] = 1 };
	end
	GameTooltip:AddDoubleLine(name, level, color.r, color.g, color.b, 1, 1, 1);

	text = "";
	if (race) then
		text = race;
	end
	if (class) then
		if (text ~= "") then
			text = text .. " ";
		end
		text = text .. class;
	end
	if (text ~= "") then
		GameTooltip:AddLine(text, 1, 1, 1);
	end

	if (ctype) then
		GameTooltip:AddLine(ctype, 1, 1, 1);
	end
	
	if (UnitIsDead(unitid)) then
		GameTooltip:AddLine("Dead");
	end

	GameTooltip:Show();
	CT_RA_CurrentMemberFrame = this;
end


-- ------------------------------------------------------------
-- Hooked function: CT_RAMenuDisplay_LockGroups (from CT_RAMenu.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RAMenuDisplay_LockGroups()
	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RAMenuDisplay_LockGroups) then
		EkCTRA_MTTargets_OldFunc.CT_RAMenuDisplay_LockGroups();
	end

	if (EkCTRA_MTTargets_Status == 0) then
		return;
	end

	-- && Added this line to fix a bug (was not hiding/showing
	-- MT and PT drag frames when you toggle the "Lock Group Positions"
	-- checkbox).
	CT_RA_UpdateRaidMovability();
end


-- ------------------------------------------------------------
-- Hooked function: CT_RAMenuDisplay_ShowGroupNames (from CT_RAMenu.lua)
-- ------------------------------------------------------------

function EkCTRA_MTTargets_CT_RAMenuDisplay_ShowGroupNames()
	-- Call the original function.
	if (EkCTRA_MTTargets_OldFunc.CT_RAMenuDisplay_ShowGroupNames) then
		EkCTRA_MTTargets_OldFunc.CT_RAMenuDisplay_ShowGroupNames();
	end

	if (EkCTRA_MTTargets_Status == 0) then
		return;
	end

	-- && Added this line to fix a bug (was not hiding/showing MT and PT
	-- group names when you toggled the "Show Group Names" checkbox).
	CT_RA_UpdateRaidFrames();
end
